
from multipers.simplex_tree_multi import SimplexTreeMulti, SimplexTreeMulti_type
import multipers
from typing import Optional,Literal
import threading
import os 
from joblib import Parallel, delayed
from warnings import warn

from multipers.slicer cimport *
from multipers.filtrations cimport *
from multipers.filtration_conversions cimport *
## TODO: these two are not needed, remove that by updating rank code.
from multipers.point_measure import sparsify, rank_decomposition_by_rectangles
from multipers.grids import compute_grid, sanitize_grid, evaluate_in_grid, _push_pts_to_line 
from multipers.array_api import api_from_tensor

import numpy as np
cimport cython
from libcpp.string cimport string
# python_value_type = np.float32
from typing import Union
from cython.operator cimport dereference

## WARNING : This is repeated in the pxd file ...
python_indices_type=np.int32
python_tensor_dtype = np.int32

global available_slicers
available_slicers = tuple((
  _KSlicer_Matrix0_vine_i32,
  _KSlicer_Matrix1_vine_i32,
  _KSlicer_Matrix0_vine_i64,
  _KSlicer_Matrix1_vine_i64,
  _KSlicer_Matrix0_vine_f32,
  _KSlicer_Matrix1_vine_f32,
  _KSlicer_Matrix0_vine_f64,
  _KSlicer_Matrix1_vine_f64,
  _Slicer_Matrix0_vine_i32,
  _Slicer_Matrix1_vine_i32,
  _Slicer_Matrix0_vine_i64,
  _Slicer_Matrix1_vine_i64,
  _Slicer_Matrix0_vine_f32,
  _Slicer_Matrix1_vine_f32,
  _Slicer_Matrix0_vine_f64,
  _Slicer_Matrix1_vine_f64,
  _KSlicer_Matrix0_i32,
  _KSlicer_Matrix1_i32,
  _KSlicer_Matrix0_i64,
  _KSlicer_Matrix1_i64,
  _KSlicer_Matrix0_f32,
  _KSlicer_Matrix1_f32,
  _KSlicer_Matrix0_f64,
  _KSlicer_Matrix1_f64,
  _Slicer_Matrix0_i32,
  _Slicer_Matrix1_i32,
  _Slicer_Matrix0_i64,
  _Slicer_Matrix1_i64,
  _Slicer_Matrix0_f32,
  _Slicer_Matrix1_f32,
  _Slicer_Matrix0_f64,
  _Slicer_Matrix1_f64,
  _KSlicer_GudhiCohomology0_i32,
  _KSlicer_GudhiCohomology0_i64,
  _KSlicer_GudhiCohomology0_f32,
  _KSlicer_GudhiCohomology0_f64,
  _Slicer_GudhiCohomology0_i32,
  _Slicer_GudhiCohomology0_i64,
  _Slicer_GudhiCohomology0_f32,
  _Slicer_GudhiCohomology0_f64,
))

global available_columns
available_columns = set((
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "INTRUSIVE_SET",
  "INTRUSIVE_SET",
  "INTRUSIVE_SET",
  "INTRUSIVE_SET",
  "INTRUSIVE_SET",
  "INTRUSIVE_SET",
  "INTRUSIVE_SET",
))

global available_dtype
available_dtype = set([
  "np.int32",
  "np.int32",
  "np.int64",
  "np.int64",
  "np.float32",
  "np.float32",
  "np.float64",
  "np.float64",
  "np.int32",
  "np.int32",
  "np.int64",
  "np.int64",
  "np.float32",
  "np.float32",
  "np.float64",
  "np.float64",
  "np.int32",
  "np.int32",
  "np.int64",
  "np.int64",
  "np.float32",
  "np.float32",
  "np.float64",
  "np.float64",
  "np.int32",
  "np.int32",
  "np.int64",
  "np.int64",
  "np.float32",
  "np.float32",
  "np.float64",
  "np.float64",
  "np.int32",
  "np.int64",
  "np.float32",
  "np.float64",
  "np.int32",
  "np.int64",
  "np.float32",
  "np.float64",
])


global available_pers_backend
available_pers_backend = set([
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "GudhiCohomology",
  "GudhiCohomology",
  "GudhiCohomology",
  "GudhiCohomology",
  "GudhiCohomology",
  "GudhiCohomology",
  "GudhiCohomology",
  "GudhiCohomology",
])



global column_type
_column_type = Literal[
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "NAIVE_VECTOR",
  "INTRUSIVE_SET",
  "INTRUSIVE_SET",
  "INTRUSIVE_SET",
  "INTRUSIVE_SET",
  "INTRUSIVE_SET",
  "INTRUSIVE_SET",
  "INTRUSIVE_SET",
  "INTRUSIVE_SET",
]

global _slicers_type
Slicer_type = Union[
  _KSlicer_Matrix0_vine_i32,
  _KSlicer_Matrix1_vine_i32,
  _KSlicer_Matrix0_vine_i64,
  _KSlicer_Matrix1_vine_i64,
  _KSlicer_Matrix0_vine_f32,
  _KSlicer_Matrix1_vine_f32,
  _KSlicer_Matrix0_vine_f64,
  _KSlicer_Matrix1_vine_f64,
  _Slicer_Matrix0_vine_i32,
  _Slicer_Matrix1_vine_i32,
  _Slicer_Matrix0_vine_i64,
  _Slicer_Matrix1_vine_i64,
  _Slicer_Matrix0_vine_f32,
  _Slicer_Matrix1_vine_f32,
  _Slicer_Matrix0_vine_f64,
  _Slicer_Matrix1_vine_f64,
  _KSlicer_Matrix0_i32,
  _KSlicer_Matrix1_i32,
  _KSlicer_Matrix0_i64,
  _KSlicer_Matrix1_i64,
  _KSlicer_Matrix0_f32,
  _KSlicer_Matrix1_f32,
  _KSlicer_Matrix0_f64,
  _KSlicer_Matrix1_f64,
  _Slicer_Matrix0_i32,
  _Slicer_Matrix1_i32,
  _Slicer_Matrix0_i64,
  _Slicer_Matrix1_i64,
  _Slicer_Matrix0_f32,
  _Slicer_Matrix1_f32,
  _Slicer_Matrix0_f64,
  _Slicer_Matrix1_f64,
  _KSlicer_GudhiCohomology0_i32,
  _KSlicer_GudhiCohomology0_i64,
  _KSlicer_GudhiCohomology0_f32,
  _KSlicer_GudhiCohomology0_f64,
  _Slicer_GudhiCohomology0_i32,
  _Slicer_GudhiCohomology0_i64,
  _Slicer_GudhiCohomology0_f32,
  _Slicer_GudhiCohomology0_f64,
]

global _valid_dtypes
_valid_dtype = Union[
  np.int32,
  np.int32,
  np.int64,
  np.int64,
  np.float32,
  np.float32,
  np.float64,
  np.float64,
  np.int32,
  np.int32,
  np.int64,
  np.int64,
  np.float32,
  np.float32,
  np.float64,
  np.float64,
  np.int32,
  np.int32,
  np.int64,
  np.int64,
  np.float32,
  np.float32,
  np.float64,
  np.float64,
  np.int32,
  np.int32,
  np.int64,
  np.int64,
  np.float32,
  np.float32,
  np.float64,
  np.float64,
  np.int32,
  np.int64,
  np.float32,
  np.float64,
  np.int32,
  np.int64,
  np.float32,
  np.float64,
]


global _valid_pers_backend
_valid_pers_backend = Literal[
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "Matrix",
  "GudhiCohomology",
  "GudhiCohomology",
  "GudhiCohomology",
  "GudhiCohomology",
  "GudhiCohomology",
  "GudhiCohomology",
  "GudhiCohomology",
  "GudhiCohomology",
]

#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix0_vine_i32:
    cdef C_KSlicer_Matrix0_vine_i32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int32) if issubclass(np.int32,np.floating) else np.iinfo(np.int32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix0_vine_i32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[int32_t]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[int32_t] cf
        cdef One_critical_filtration[int32_t] inf
        inf[0] = -inf[0]
        cdef int32_t[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.int32 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_i32(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix0_vine_i32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix0_vine_i32|tuple[_KSlicer_Matrix0_vine_i32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix0_vine_i32()
        cdef pair[C_KSlicer_Matrix0_vine_i32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix0_vine_i32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix0_vine_i32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix0_vine_i32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix0_vine_i32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix0_vine_i32:
        if len(self) == 0:
            return _KSlicer_Matrix0_vine_i32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix0_vine_i32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int32) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix0_vine_i32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix0_vine_i32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int32_t, int32_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix0_vine_i32._inf_value()),  dtype=np.dtype((np.int32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int32)
      cdef int32_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int32_t]] c_basepoints
        cdef vector[pair[vector[int32_t], vector[int32_t]]] c_truc
        cdef vector[vector[vector[pair[int32_t, int32_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int32) if len(y)>0 else np.empty((0,2), dtype = np.int32)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix0_vine_i32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix0_vine_i32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix0_vine_i32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int32_t],One_critical_filtration[int32_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int32_t, ndim=1] a = _ff21cview_i32(&box.first)
        cdef cnp.ndarray[int32_t, ndim=1] b = _ff21cview_i32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int32_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_i32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix0_vine_i32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix0_vine_i32|_KSlicer_Matrix0_vine_i32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int32_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix0_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.int32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int32
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_KSlicer_Matrix0_vine_i32:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix1_vine_i32:
    cdef C_KSlicer_Matrix1_vine_i32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int32) if issubclass(np.int32,np.floating) else np.iinfo(np.int32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix1_vine_i32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[int32_t]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[int32_t] cf
        cdef One_critical_filtration[int32_t] inf
        inf[0] = -inf[0]
        cdef int32_t[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.int32 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_i32(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix1_vine_i32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix1_vine_i32|tuple[_KSlicer_Matrix1_vine_i32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix1_vine_i32()
        cdef pair[C_KSlicer_Matrix1_vine_i32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix1_vine_i32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix1_vine_i32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix1_vine_i32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix1_vine_i32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix1_vine_i32:
        if len(self) == 0:
            return _KSlicer_Matrix1_vine_i32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix1_vine_i32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int32) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix1_vine_i32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix1_vine_i32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int32_t, int32_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix1_vine_i32._inf_value()),  dtype=np.dtype((np.int32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int32)
      cdef int32_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int32_t]] c_basepoints
        cdef vector[pair[vector[int32_t], vector[int32_t]]] c_truc
        cdef vector[vector[vector[pair[int32_t, int32_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int32) if len(y)>0 else np.empty((0,2), dtype = np.int32)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix1_vine_i32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix1_vine_i32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix1_vine_i32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int32_t],One_critical_filtration[int32_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int32_t, ndim=1] a = _ff21cview_i32(&box.first)
        cdef cnp.ndarray[int32_t, ndim=1] b = _ff21cview_i32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int32_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_i32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix1_vine_i32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix1_vine_i32|_KSlicer_Matrix1_vine_i32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int32_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix1_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.int32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int32
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_KSlicer_Matrix1_vine_i32:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix0_vine_i64:
    cdef C_KSlicer_Matrix0_vine_i64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int64) if issubclass(np.int64,np.floating) else np.iinfo(np.int64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix0_vine_i64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[int64_t]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[int64_t] cf
        cdef One_critical_filtration[int64_t] inf
        inf[0] = -inf[0]
        cdef int64_t[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.int64 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_i64(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix0_vine_i64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix0_vine_i64|tuple[_KSlicer_Matrix0_vine_i64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix0_vine_i64()
        cdef pair[C_KSlicer_Matrix0_vine_i64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix0_vine_i64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix0_vine_i64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix0_vine_i64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix0_vine_i64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix0_vine_i64:
        if len(self) == 0:
            return _KSlicer_Matrix0_vine_i64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix0_vine_i64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int64) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix0_vine_i64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix0_vine_i64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int64_t, int64_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix0_vine_i64._inf_value()),  dtype=np.dtype((np.int64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int64)
      cdef int64_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int64_t]] c_basepoints
        cdef vector[pair[vector[int64_t], vector[int64_t]]] c_truc
        cdef vector[vector[vector[pair[int64_t, int64_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int64) if len(y)>0 else np.empty((0,2), dtype = np.int64)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix0_vine_i64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix0_vine_i64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix0_vine_i64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int64_t],One_critical_filtration[int64_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int64_t, ndim=1] a = _ff21cview_i64(&box.first)
        cdef cnp.ndarray[int64_t, ndim=1] b = _ff21cview_i64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int64_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_i64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix0_vine_i64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix0_vine_i32|_KSlicer_Matrix0_vine_i64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int64_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix0_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.int64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int64
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_KSlicer_Matrix0_vine_i64:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix1_vine_i64:
    cdef C_KSlicer_Matrix1_vine_i64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int64) if issubclass(np.int64,np.floating) else np.iinfo(np.int64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix1_vine_i64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[int64_t]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[int64_t] cf
        cdef One_critical_filtration[int64_t] inf
        inf[0] = -inf[0]
        cdef int64_t[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.int64 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_i64(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix1_vine_i64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix1_vine_i64|tuple[_KSlicer_Matrix1_vine_i64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix1_vine_i64()
        cdef pair[C_KSlicer_Matrix1_vine_i64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix1_vine_i64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix1_vine_i64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix1_vine_i64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix1_vine_i64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix1_vine_i64:
        if len(self) == 0:
            return _KSlicer_Matrix1_vine_i64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix1_vine_i64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int64) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix1_vine_i64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix1_vine_i64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int64_t, int64_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix1_vine_i64._inf_value()),  dtype=np.dtype((np.int64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int64)
      cdef int64_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int64_t]] c_basepoints
        cdef vector[pair[vector[int64_t], vector[int64_t]]] c_truc
        cdef vector[vector[vector[pair[int64_t, int64_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int64) if len(y)>0 else np.empty((0,2), dtype = np.int64)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix1_vine_i64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix1_vine_i64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix1_vine_i64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int64_t],One_critical_filtration[int64_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int64_t, ndim=1] a = _ff21cview_i64(&box.first)
        cdef cnp.ndarray[int64_t, ndim=1] b = _ff21cview_i64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int64_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_i64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix1_vine_i64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix1_vine_i32|_KSlicer_Matrix1_vine_i64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int64_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix1_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.int64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int64
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_KSlicer_Matrix1_vine_i64:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix0_vine_f32:
    cdef C_KSlicer_Matrix0_vine_f32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float32) if issubclass(np.float32,np.floating) else np.iinfo(np.float32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix0_vine_f32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[float]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[float] cf
        cdef One_critical_filtration[float] inf
        inf[0] = -inf[0]
        cdef float[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.float32 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_f32(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix0_vine_f32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix0_vine_f32|tuple[_KSlicer_Matrix0_vine_f32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix0_vine_f32()
        cdef pair[C_KSlicer_Matrix0_vine_f32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix0_vine_f32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix0_vine_f32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix0_vine_f32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix0_vine_f32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix0_vine_f32:
        if len(self) == 0:
            return _KSlicer_Matrix0_vine_f32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix0_vine_f32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix0_vine_f32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix0_vine_f32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float32)
        cdef Line[float] line
        if direction is None:
            line = Line[float](_py21c_f32(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float32)
            line = Line[float](_py21c_f32(basepoint),_py21c_f32(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[float, float]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix0_vine_f32._inf_value()),  dtype=np.dtype((np.float32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix0_vine_f32._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _KSlicer_Matrix0_vine_f32._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float32)
      cdef float[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[float]] c_basepoints
        cdef vector[pair[vector[float], vector[float]]] c_truc
        cdef vector[vector[vector[pair[float, float]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float32) if len(y)>0 else np.empty((0,2), dtype = np.float32)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix0_vine_f32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix0_vine_f32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix0_vine_f32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[float],One_critical_filtration[float]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[float, ndim=1] a = _ff21cview_f32(&box.first)
        cdef cnp.ndarray[float, ndim=1] b = _ff21cview_f32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[float]] v = self.truc.get_filtration_values()
        out = _vff21cview_f32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_f32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix0_vine_f32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix0_vine_i32|_KSlicer_Matrix0_vine_f32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[float]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix0_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.float32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float32
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_KSlicer_Matrix0_vine_f32:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix1_vine_f32:
    cdef C_KSlicer_Matrix1_vine_f32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float32) if issubclass(np.float32,np.floating) else np.iinfo(np.float32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix1_vine_f32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[float]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[float] cf
        cdef One_critical_filtration[float] inf
        inf[0] = -inf[0]
        cdef float[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.float32 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_f32(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix1_vine_f32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix1_vine_f32|tuple[_KSlicer_Matrix1_vine_f32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix1_vine_f32()
        cdef pair[C_KSlicer_Matrix1_vine_f32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix1_vine_f32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix1_vine_f32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix1_vine_f32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix1_vine_f32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix1_vine_f32:
        if len(self) == 0:
            return _KSlicer_Matrix1_vine_f32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix1_vine_f32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix1_vine_f32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix1_vine_f32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float32)
        cdef Line[float] line
        if direction is None:
            line = Line[float](_py21c_f32(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float32)
            line = Line[float](_py21c_f32(basepoint),_py21c_f32(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[float, float]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix1_vine_f32._inf_value()),  dtype=np.dtype((np.float32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix1_vine_f32._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _KSlicer_Matrix1_vine_f32._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float32)
      cdef float[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[float]] c_basepoints
        cdef vector[pair[vector[float], vector[float]]] c_truc
        cdef vector[vector[vector[pair[float, float]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float32) if len(y)>0 else np.empty((0,2), dtype = np.float32)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix1_vine_f32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix1_vine_f32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix1_vine_f32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[float],One_critical_filtration[float]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[float, ndim=1] a = _ff21cview_f32(&box.first)
        cdef cnp.ndarray[float, ndim=1] b = _ff21cview_f32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[float]] v = self.truc.get_filtration_values()
        out = _vff21cview_f32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_f32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix1_vine_f32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix1_vine_i32|_KSlicer_Matrix1_vine_f32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[float]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix1_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.float32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float32
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_KSlicer_Matrix1_vine_f32:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix0_vine_f64:
    cdef C_KSlicer_Matrix0_vine_f64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float64) if issubclass(np.float64,np.floating) else np.iinfo(np.float64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix0_vine_f64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[double]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[double] cf
        cdef One_critical_filtration[double] inf
        inf[0] = -inf[0]
        cdef double[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.float64 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_f64(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix0_vine_f64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix0_vine_f64|tuple[_KSlicer_Matrix0_vine_f64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix0_vine_f64()
        cdef pair[C_KSlicer_Matrix0_vine_f64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix0_vine_f64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix0_vine_f64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix0_vine_f64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix0_vine_f64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix0_vine_f64:
        if len(self) == 0:
            return _KSlicer_Matrix0_vine_f64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix0_vine_f64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix0_vine_f64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix0_vine_f64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float64)
        cdef Line[double] line
        if direction is None:
            line = Line[double](_py21c_f64(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float64)
            line = Line[double](_py21c_f64(basepoint),_py21c_f64(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[double, double]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix0_vine_f64._inf_value()),  dtype=np.dtype((np.float64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix0_vine_f64._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _KSlicer_Matrix0_vine_f64._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float64)
      cdef double[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[double]] c_basepoints
        cdef vector[pair[vector[double], vector[double]]] c_truc
        cdef vector[vector[vector[pair[double, double]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float64) if len(y)>0 else np.empty((0,2), dtype = np.float64)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix0_vine_f64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix0_vine_f64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix0_vine_f64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[double],One_critical_filtration[double]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[double, ndim=1] a = _ff21cview_f64(&box.first)
        cdef cnp.ndarray[double, ndim=1] b = _ff21cview_f64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[double]] v = self.truc.get_filtration_values()
        out = _vff21cview_f64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_f64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix0_vine_f64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix0_vine_i32|_KSlicer_Matrix0_vine_f64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[double]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix0_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.float64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float64
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_KSlicer_Matrix0_vine_f64:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix1_vine_f64:
    cdef C_KSlicer_Matrix1_vine_f64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float64) if issubclass(np.float64,np.floating) else np.iinfo(np.float64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix1_vine_f64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[double]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[double] cf
        cdef One_critical_filtration[double] inf
        inf[0] = -inf[0]
        cdef double[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.float64 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_f64(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix1_vine_f64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix1_vine_f64|tuple[_KSlicer_Matrix1_vine_f64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix1_vine_f64()
        cdef pair[C_KSlicer_Matrix1_vine_f64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix1_vine_f64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix1_vine_f64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix1_vine_f64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix1_vine_f64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix1_vine_f64:
        if len(self) == 0:
            return _KSlicer_Matrix1_vine_f64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix1_vine_f64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix1_vine_f64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix1_vine_f64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float64)
        cdef Line[double] line
        if direction is None:
            line = Line[double](_py21c_f64(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float64)
            line = Line[double](_py21c_f64(basepoint),_py21c_f64(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[double, double]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix1_vine_f64._inf_value()),  dtype=np.dtype((np.float64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix1_vine_f64._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _KSlicer_Matrix1_vine_f64._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float64)
      cdef double[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[double]] c_basepoints
        cdef vector[pair[vector[double], vector[double]]] c_truc
        cdef vector[vector[vector[pair[double, double]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float64) if len(y)>0 else np.empty((0,2), dtype = np.float64)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix1_vine_f64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix1_vine_f64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix1_vine_f64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[double],One_critical_filtration[double]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[double, ndim=1] a = _ff21cview_f64(&box.first)
        cdef cnp.ndarray[double, ndim=1] b = _ff21cview_f64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[double]] v = self.truc.get_filtration_values()
        out = _vff21cview_f64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_f64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix1_vine_f64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix1_vine_i32|_KSlicer_Matrix1_vine_f64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[double]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix1_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.float64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float64
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_KSlicer_Matrix1_vine_f64:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _Slicer_Matrix0_vine_i32:
    cdef C_Slicer_Matrix0_vine_i32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int32) if issubclass(np.int32,np.floating) else np.iinfo(np.int32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix0_vine_i32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.int32 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[int32_t]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_i32(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix0_vine_i32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix0_vine_i32|tuple[_Slicer_Matrix0_vine_i32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix0_vine_i32()
        cdef pair[C_Slicer_Matrix0_vine_i32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix0_vine_i32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix0_vine_i32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix0_vine_i32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix0_vine_i32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix0_vine_i32:
        if len(self) == 0:
            return _Slicer_Matrix0_vine_i32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix0_vine_i32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int32) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix0_vine_i32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix0_vine_i32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int32_t, int32_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix0_vine_i32._inf_value()),  dtype=np.dtype((np.int32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int32)
      cdef int32_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int32_t]] c_basepoints
        cdef vector[pair[vector[int32_t], vector[int32_t]]] c_truc
        cdef vector[vector[vector[pair[int32_t, int32_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int32) if len(y)>0 else np.empty((0,2), dtype = np.int32)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix0_vine_i32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix0_vine_i32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix0_vine_i32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int32_t],One_critical_filtration[int32_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int32_t, ndim=1] a = _ff21cview_i32(&box.first)
        cdef cnp.ndarray[int32_t, ndim=1] b = _ff21cview_i32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int32_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_i32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix0_vine_i32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix0_vine_i32|_Slicer_Matrix0_vine_i32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int32_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix0_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.int32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int32
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_Slicer_Matrix0_vine_i32:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix0_vine_i32:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix0_vine_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix0_vine_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_Matrix1_vine_i32:
    cdef C_Slicer_Matrix1_vine_i32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int32) if issubclass(np.int32,np.floating) else np.iinfo(np.int32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix1_vine_i32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.int32 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[int32_t]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_i32(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix1_vine_i32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix1_vine_i32|tuple[_Slicer_Matrix1_vine_i32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix1_vine_i32()
        cdef pair[C_Slicer_Matrix1_vine_i32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix1_vine_i32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix1_vine_i32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix1_vine_i32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix1_vine_i32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix1_vine_i32:
        if len(self) == 0:
            return _Slicer_Matrix1_vine_i32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix1_vine_i32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int32) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix1_vine_i32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix1_vine_i32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int32_t, int32_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix1_vine_i32._inf_value()),  dtype=np.dtype((np.int32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int32)
      cdef int32_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int32_t]] c_basepoints
        cdef vector[pair[vector[int32_t], vector[int32_t]]] c_truc
        cdef vector[vector[vector[pair[int32_t, int32_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int32) if len(y)>0 else np.empty((0,2), dtype = np.int32)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix1_vine_i32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix1_vine_i32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix1_vine_i32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int32_t],One_critical_filtration[int32_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int32_t, ndim=1] a = _ff21cview_i32(&box.first)
        cdef cnp.ndarray[int32_t, ndim=1] b = _ff21cview_i32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int32_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_i32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix1_vine_i32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix1_vine_i32|_Slicer_Matrix1_vine_i32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int32_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix1_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.int32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int32
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_Slicer_Matrix1_vine_i32:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix1_vine_i32:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix1_vine_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix1_vine_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_Matrix0_vine_i64:
    cdef C_Slicer_Matrix0_vine_i64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int64) if issubclass(np.int64,np.floating) else np.iinfo(np.int64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix0_vine_i64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.int64 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[int64_t]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_i64(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix0_vine_i64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix0_vine_i64|tuple[_Slicer_Matrix0_vine_i64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix0_vine_i64()
        cdef pair[C_Slicer_Matrix0_vine_i64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix0_vine_i64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix0_vine_i64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix0_vine_i64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix0_vine_i64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix0_vine_i64:
        if len(self) == 0:
            return _Slicer_Matrix0_vine_i64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix0_vine_i64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int64) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix0_vine_i64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix0_vine_i64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int64_t, int64_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix0_vine_i64._inf_value()),  dtype=np.dtype((np.int64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int64)
      cdef int64_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int64_t]] c_basepoints
        cdef vector[pair[vector[int64_t], vector[int64_t]]] c_truc
        cdef vector[vector[vector[pair[int64_t, int64_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int64) if len(y)>0 else np.empty((0,2), dtype = np.int64)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix0_vine_i64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix0_vine_i64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix0_vine_i64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int64_t],One_critical_filtration[int64_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int64_t, ndim=1] a = _ff21cview_i64(&box.first)
        cdef cnp.ndarray[int64_t, ndim=1] b = _ff21cview_i64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int64_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_i64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix0_vine_i64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix0_vine_i32|_Slicer_Matrix0_vine_i64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int64_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix0_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.int64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int64
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_Slicer_Matrix0_vine_i64:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix0_vine_i64:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix0_vine_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix0_vine_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_Matrix1_vine_i64:
    cdef C_Slicer_Matrix1_vine_i64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int64) if issubclass(np.int64,np.floating) else np.iinfo(np.int64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix1_vine_i64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.int64 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[int64_t]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_i64(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix1_vine_i64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix1_vine_i64|tuple[_Slicer_Matrix1_vine_i64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix1_vine_i64()
        cdef pair[C_Slicer_Matrix1_vine_i64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix1_vine_i64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix1_vine_i64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix1_vine_i64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix1_vine_i64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix1_vine_i64:
        if len(self) == 0:
            return _Slicer_Matrix1_vine_i64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix1_vine_i64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int64) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix1_vine_i64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix1_vine_i64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int64_t, int64_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix1_vine_i64._inf_value()),  dtype=np.dtype((np.int64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int64)
      cdef int64_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int64_t]] c_basepoints
        cdef vector[pair[vector[int64_t], vector[int64_t]]] c_truc
        cdef vector[vector[vector[pair[int64_t, int64_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int64) if len(y)>0 else np.empty((0,2), dtype = np.int64)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix1_vine_i64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix1_vine_i64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix1_vine_i64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int64_t],One_critical_filtration[int64_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int64_t, ndim=1] a = _ff21cview_i64(&box.first)
        cdef cnp.ndarray[int64_t, ndim=1] b = _ff21cview_i64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int64_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_i64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix1_vine_i64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix1_vine_i32|_Slicer_Matrix1_vine_i64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int64_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix1_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.int64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int64
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_Slicer_Matrix1_vine_i64:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix1_vine_i64:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix1_vine_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix1_vine_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_Matrix0_vine_f32:
    cdef C_Slicer_Matrix0_vine_f32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float32) if issubclass(np.float32,np.floating) else np.iinfo(np.float32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix0_vine_f32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.float32 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[float]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_f32(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix0_vine_f32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix0_vine_f32|tuple[_Slicer_Matrix0_vine_f32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix0_vine_f32()
        cdef pair[C_Slicer_Matrix0_vine_f32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix0_vine_f32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix0_vine_f32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix0_vine_f32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix0_vine_f32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix0_vine_f32:
        if len(self) == 0:
            return _Slicer_Matrix0_vine_f32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix0_vine_f32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix0_vine_f32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix0_vine_f32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float32)
        cdef Line[float] line
        if direction is None:
            line = Line[float](_py21c_f32(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float32)
            line = Line[float](_py21c_f32(basepoint),_py21c_f32(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[float, float]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix0_vine_f32._inf_value()),  dtype=np.dtype((np.float32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix0_vine_f32._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _Slicer_Matrix0_vine_f32._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float32)
      cdef float[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[float]] c_basepoints
        cdef vector[pair[vector[float], vector[float]]] c_truc
        cdef vector[vector[vector[pair[float, float]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float32) if len(y)>0 else np.empty((0,2), dtype = np.float32)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix0_vine_f32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix0_vine_f32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix0_vine_f32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[float],One_critical_filtration[float]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[float, ndim=1] a = _ff21cview_f32(&box.first)
        cdef cnp.ndarray[float, ndim=1] b = _ff21cview_f32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[float]] v = self.truc.get_filtration_values()
        out = _vff21cview_f32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_f32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix0_vine_f32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix0_vine_i32|_Slicer_Matrix0_vine_f32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[float]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix0_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.float32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float32
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_Slicer_Matrix0_vine_f32:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix0_vine_f32:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix0_vine_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix0_vine_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_Matrix1_vine_f32:
    cdef C_Slicer_Matrix1_vine_f32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float32) if issubclass(np.float32,np.floating) else np.iinfo(np.float32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix1_vine_f32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.float32 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[float]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_f32(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix1_vine_f32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix1_vine_f32|tuple[_Slicer_Matrix1_vine_f32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix1_vine_f32()
        cdef pair[C_Slicer_Matrix1_vine_f32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix1_vine_f32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix1_vine_f32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix1_vine_f32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix1_vine_f32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix1_vine_f32:
        if len(self) == 0:
            return _Slicer_Matrix1_vine_f32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix1_vine_f32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix1_vine_f32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix1_vine_f32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float32)
        cdef Line[float] line
        if direction is None:
            line = Line[float](_py21c_f32(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float32)
            line = Line[float](_py21c_f32(basepoint),_py21c_f32(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[float, float]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix1_vine_f32._inf_value()),  dtype=np.dtype((np.float32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix1_vine_f32._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _Slicer_Matrix1_vine_f32._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float32)
      cdef float[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[float]] c_basepoints
        cdef vector[pair[vector[float], vector[float]]] c_truc
        cdef vector[vector[vector[pair[float, float]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float32) if len(y)>0 else np.empty((0,2), dtype = np.float32)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix1_vine_f32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix1_vine_f32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix1_vine_f32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[float],One_critical_filtration[float]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[float, ndim=1] a = _ff21cview_f32(&box.first)
        cdef cnp.ndarray[float, ndim=1] b = _ff21cview_f32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[float]] v = self.truc.get_filtration_values()
        out = _vff21cview_f32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_f32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix1_vine_f32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix1_vine_i32|_Slicer_Matrix1_vine_f32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[float]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix1_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.float32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float32
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_Slicer_Matrix1_vine_f32:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix1_vine_f32:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix1_vine_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix1_vine_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_Matrix0_vine_f64:
    cdef C_Slicer_Matrix0_vine_f64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float64) if issubclass(np.float64,np.floating) else np.iinfo(np.float64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix0_vine_f64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.float64 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[double]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_f64(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix0_vine_f64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix0_vine_f64|tuple[_Slicer_Matrix0_vine_f64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix0_vine_f64()
        cdef pair[C_Slicer_Matrix0_vine_f64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix0_vine_f64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix0_vine_f64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix0_vine_f64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix0_vine_f64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix0_vine_f64:
        if len(self) == 0:
            return _Slicer_Matrix0_vine_f64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix0_vine_f64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix0_vine_f64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix0_vine_f64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float64)
        cdef Line[double] line
        if direction is None:
            line = Line[double](_py21c_f64(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float64)
            line = Line[double](_py21c_f64(basepoint),_py21c_f64(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[double, double]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix0_vine_f64._inf_value()),  dtype=np.dtype((np.float64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix0_vine_f64._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _Slicer_Matrix0_vine_f64._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float64)
      cdef double[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[double]] c_basepoints
        cdef vector[pair[vector[double], vector[double]]] c_truc
        cdef vector[vector[vector[pair[double, double]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float64) if len(y)>0 else np.empty((0,2), dtype = np.float64)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix0_vine_f64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix0_vine_f64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix0_vine_f64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[double],One_critical_filtration[double]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[double, ndim=1] a = _ff21cview_f64(&box.first)
        cdef cnp.ndarray[double, ndim=1] b = _ff21cview_f64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[double]] v = self.truc.get_filtration_values()
        out = _vff21cview_f64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_f64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix0_vine_f64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix0_vine_i32|_Slicer_Matrix0_vine_f64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[double]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix0_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.float64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float64
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_Slicer_Matrix0_vine_f64:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix0_vine_f64:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix0_vine_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix0_vine_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_Matrix1_vine_f64:
    cdef C_Slicer_Matrix1_vine_f64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float64) if issubclass(np.float64,np.floating) else np.iinfo(np.float64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix1_vine_f64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.float64 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[double]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_f64(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix1_vine_f64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix1_vine_f64|tuple[_Slicer_Matrix1_vine_f64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix1_vine_f64()
        cdef pair[C_Slicer_Matrix1_vine_f64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix1_vine_f64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix1_vine_f64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix1_vine_f64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix1_vine_f64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix1_vine_f64:
        if len(self) == 0:
            return _Slicer_Matrix1_vine_f64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix1_vine_f64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix1_vine_f64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix1_vine_f64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float64)
        cdef Line[double] line
        if direction is None:
            line = Line[double](_py21c_f64(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float64)
            line = Line[double](_py21c_f64(basepoint),_py21c_f64(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[double, double]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix1_vine_f64._inf_value()),  dtype=np.dtype((np.float64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix1_vine_f64._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _Slicer_Matrix1_vine_f64._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float64)
      cdef double[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[double]] c_basepoints
        cdef vector[pair[vector[double], vector[double]]] c_truc
        cdef vector[vector[vector[pair[double, double]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float64) if len(y)>0 else np.empty((0,2), dtype = np.float64)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix1_vine_f64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix1_vine_f64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix1_vine_f64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[double],One_critical_filtration[double]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[double, ndim=1] a = _ff21cview_f64(&box.first)
        cdef cnp.ndarray[double, ndim=1] b = _ff21cview_f64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[double]] v = self.truc.get_filtration_values()
        out = _vff21cview_f64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_f64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix1_vine_f64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix1_vine_i32|_Slicer_Matrix1_vine_f64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[double]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix1_vine_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] = None,
        dtype = np.float64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float64
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return True
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"

    def vine_update(self,basepoint,direction=None)->_Slicer_Matrix1_vine_f64:
        """
        Updates the barcode, on a line, using the vineyard algorithm.
        """
        self.push_to_line(basepoint,direction)
        self.truc.vineyard_update()
        return self
    def get_representative_cycles(self, bool update=True, bool detailed=False):
        """
        Returns the representative cycles of the current barcode.
        Recomputes the generators if update=True
        """
        return self.truc.get_representative_cycles(update, detailed)
    def get_permutation(self):
        """
        Returns the current generator permutation (w.r.t. vineyard).
        """
        return self.truc.get_current_order()

    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix1_vine_f64:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix1_vine_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix1_vine_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix0_i32:
    cdef C_KSlicer_Matrix0_i32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int32) if issubclass(np.int32,np.floating) else np.iinfo(np.int32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix0_i32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[int32_t]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[int32_t] cf
        cdef One_critical_filtration[int32_t] inf
        inf[0] = -inf[0]
        cdef int32_t[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.int32 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_i32(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix0_i32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix0_i32|tuple[_KSlicer_Matrix0_i32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix0_i32()
        cdef pair[C_KSlicer_Matrix0_i32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix0_i32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix0_i32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix0_i32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix0_i32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix0_i32:
        if len(self) == 0:
            return _KSlicer_Matrix0_i32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix0_i32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int32) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix0_i32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix0_i32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int32_t, int32_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix0_i32._inf_value()),  dtype=np.dtype((np.int32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int32)
      cdef int32_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int32_t]] c_basepoints
        cdef vector[pair[vector[int32_t], vector[int32_t]]] c_truc
        cdef vector[vector[vector[pair[int32_t, int32_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int32) if len(y)>0 else np.empty((0,2), dtype = np.int32)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix0_i32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix0_i32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix0_i32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int32_t],One_critical_filtration[int32_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int32_t, ndim=1] a = _ff21cview_i32(&box.first)
        cdef cnp.ndarray[int32_t, ndim=1] b = _ff21cview_i32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int32_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_i32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix0_i32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix0_i32|_KSlicer_Matrix0_i32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int32_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.int32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int32
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix1_i32:
    cdef C_KSlicer_Matrix1_i32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int32) if issubclass(np.int32,np.floating) else np.iinfo(np.int32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix1_i32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[int32_t]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[int32_t] cf
        cdef One_critical_filtration[int32_t] inf
        inf[0] = -inf[0]
        cdef int32_t[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.int32 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_i32(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix1_i32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix1_i32|tuple[_KSlicer_Matrix1_i32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix1_i32()
        cdef pair[C_KSlicer_Matrix1_i32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix1_i32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix1_i32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix1_i32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix1_i32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix1_i32:
        if len(self) == 0:
            return _KSlicer_Matrix1_i32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix1_i32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int32) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix1_i32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix1_i32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int32_t, int32_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix1_i32._inf_value()),  dtype=np.dtype((np.int32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int32)
      cdef int32_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int32_t]] c_basepoints
        cdef vector[pair[vector[int32_t], vector[int32_t]]] c_truc
        cdef vector[vector[vector[pair[int32_t, int32_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int32) if len(y)>0 else np.empty((0,2), dtype = np.int32)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix1_i32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix1_i32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix1_i32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int32_t],One_critical_filtration[int32_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int32_t, ndim=1] a = _ff21cview_i32(&box.first)
        cdef cnp.ndarray[int32_t, ndim=1] b = _ff21cview_i32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int32_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_i32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix1_i32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix1_i32|_KSlicer_Matrix1_i32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int32_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix1_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.int32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int32
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix0_i64:
    cdef C_KSlicer_Matrix0_i64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int64) if issubclass(np.int64,np.floating) else np.iinfo(np.int64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix0_i64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[int64_t]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[int64_t] cf
        cdef One_critical_filtration[int64_t] inf
        inf[0] = -inf[0]
        cdef int64_t[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.int64 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_i64(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix0_i64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix0_i64|tuple[_KSlicer_Matrix0_i64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix0_i64()
        cdef pair[C_KSlicer_Matrix0_i64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix0_i64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix0_i64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix0_i64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix0_i64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix0_i64:
        if len(self) == 0:
            return _KSlicer_Matrix0_i64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix0_i64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int64) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix0_i64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix0_i64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int64_t, int64_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix0_i64._inf_value()),  dtype=np.dtype((np.int64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int64)
      cdef int64_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int64_t]] c_basepoints
        cdef vector[pair[vector[int64_t], vector[int64_t]]] c_truc
        cdef vector[vector[vector[pair[int64_t, int64_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int64) if len(y)>0 else np.empty((0,2), dtype = np.int64)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix0_i64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix0_i64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix0_i64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int64_t],One_critical_filtration[int64_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int64_t, ndim=1] a = _ff21cview_i64(&box.first)
        cdef cnp.ndarray[int64_t, ndim=1] b = _ff21cview_i64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int64_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_i64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix0_i64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix0_i32|_KSlicer_Matrix0_i64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int64_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.int64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int64
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix1_i64:
    cdef C_KSlicer_Matrix1_i64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int64) if issubclass(np.int64,np.floating) else np.iinfo(np.int64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix1_i64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[int64_t]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[int64_t] cf
        cdef One_critical_filtration[int64_t] inf
        inf[0] = -inf[0]
        cdef int64_t[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.int64 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_i64(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix1_i64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix1_i64|tuple[_KSlicer_Matrix1_i64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix1_i64()
        cdef pair[C_KSlicer_Matrix1_i64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix1_i64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix1_i64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix1_i64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix1_i64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix1_i64:
        if len(self) == 0:
            return _KSlicer_Matrix1_i64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix1_i64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int64) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix1_i64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix1_i64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int64_t, int64_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix1_i64._inf_value()),  dtype=np.dtype((np.int64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int64)
      cdef int64_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int64_t]] c_basepoints
        cdef vector[pair[vector[int64_t], vector[int64_t]]] c_truc
        cdef vector[vector[vector[pair[int64_t, int64_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int64) if len(y)>0 else np.empty((0,2), dtype = np.int64)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix1_i64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix1_i64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix1_i64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int64_t],One_critical_filtration[int64_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int64_t, ndim=1] a = _ff21cview_i64(&box.first)
        cdef cnp.ndarray[int64_t, ndim=1] b = _ff21cview_i64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int64_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_i64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix1_i64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix1_i32|_KSlicer_Matrix1_i64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int64_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix1_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.int64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int64
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix0_f32:
    cdef C_KSlicer_Matrix0_f32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float32) if issubclass(np.float32,np.floating) else np.iinfo(np.float32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix0_f32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[float]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[float] cf
        cdef One_critical_filtration[float] inf
        inf[0] = -inf[0]
        cdef float[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.float32 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_f32(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix0_f32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix0_f32|tuple[_KSlicer_Matrix0_f32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix0_f32()
        cdef pair[C_KSlicer_Matrix0_f32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix0_f32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix0_f32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix0_f32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix0_f32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix0_f32:
        if len(self) == 0:
            return _KSlicer_Matrix0_f32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix0_f32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix0_f32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix0_f32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float32)
        cdef Line[float] line
        if direction is None:
            line = Line[float](_py21c_f32(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float32)
            line = Line[float](_py21c_f32(basepoint),_py21c_f32(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[float, float]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix0_f32._inf_value()),  dtype=np.dtype((np.float32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix0_f32._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _KSlicer_Matrix0_f32._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float32)
      cdef float[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[float]] c_basepoints
        cdef vector[pair[vector[float], vector[float]]] c_truc
        cdef vector[vector[vector[pair[float, float]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float32) if len(y)>0 else np.empty((0,2), dtype = np.float32)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix0_f32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix0_f32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix0_f32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[float],One_critical_filtration[float]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[float, ndim=1] a = _ff21cview_f32(&box.first)
        cdef cnp.ndarray[float, ndim=1] b = _ff21cview_f32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[float]] v = self.truc.get_filtration_values()
        out = _vff21cview_f32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_f32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix0_f32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix0_i32|_KSlicer_Matrix0_f32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[float]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.float32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float32
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix1_f32:
    cdef C_KSlicer_Matrix1_f32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float32) if issubclass(np.float32,np.floating) else np.iinfo(np.float32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix1_f32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[float]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[float] cf
        cdef One_critical_filtration[float] inf
        inf[0] = -inf[0]
        cdef float[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.float32 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_f32(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix1_f32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix1_f32|tuple[_KSlicer_Matrix1_f32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix1_f32()
        cdef pair[C_KSlicer_Matrix1_f32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix1_f32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix1_f32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix1_f32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix1_f32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix1_f32:
        if len(self) == 0:
            return _KSlicer_Matrix1_f32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix1_f32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix1_f32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix1_f32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float32)
        cdef Line[float] line
        if direction is None:
            line = Line[float](_py21c_f32(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float32)
            line = Line[float](_py21c_f32(basepoint),_py21c_f32(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[float, float]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix1_f32._inf_value()),  dtype=np.dtype((np.float32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix1_f32._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _KSlicer_Matrix1_f32._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float32)
      cdef float[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[float]] c_basepoints
        cdef vector[pair[vector[float], vector[float]]] c_truc
        cdef vector[vector[vector[pair[float, float]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float32) if len(y)>0 else np.empty((0,2), dtype = np.float32)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix1_f32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix1_f32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix1_f32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[float],One_critical_filtration[float]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[float, ndim=1] a = _ff21cview_f32(&box.first)
        cdef cnp.ndarray[float, ndim=1] b = _ff21cview_f32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[float]] v = self.truc.get_filtration_values()
        out = _vff21cview_f32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_f32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix1_f32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix1_i32|_KSlicer_Matrix1_f32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[float]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix1_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.float32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float32
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix0_f64:
    cdef C_KSlicer_Matrix0_f64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float64) if issubclass(np.float64,np.floating) else np.iinfo(np.float64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix0_f64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[double]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[double] cf
        cdef One_critical_filtration[double] inf
        inf[0] = -inf[0]
        cdef double[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.float64 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_f64(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix0_f64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix0_f64|tuple[_KSlicer_Matrix0_f64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix0_f64()
        cdef pair[C_KSlicer_Matrix0_f64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix0_f64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix0_f64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix0_f64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix0_f64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix0_f64:
        if len(self) == 0:
            return _KSlicer_Matrix0_f64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix0_f64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix0_f64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix0_f64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float64)
        cdef Line[double] line
        if direction is None:
            line = Line[double](_py21c_f64(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float64)
            line = Line[double](_py21c_f64(basepoint),_py21c_f64(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[double, double]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix0_f64._inf_value()),  dtype=np.dtype((np.float64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix0_f64._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _KSlicer_Matrix0_f64._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float64)
      cdef double[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[double]] c_basepoints
        cdef vector[pair[vector[double], vector[double]]] c_truc
        cdef vector[vector[vector[pair[double, double]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float64) if len(y)>0 else np.empty((0,2), dtype = np.float64)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix0_f64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix0_f64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix0_f64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[double],One_critical_filtration[double]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[double, ndim=1] a = _ff21cview_f64(&box.first)
        cdef cnp.ndarray[double, ndim=1] b = _ff21cview_f64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[double]] v = self.truc.get_filtration_values()
        out = _vff21cview_f64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_f64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix0_f64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix0_i32|_KSlicer_Matrix0_f64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[double]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.float64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float64
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_Matrix1_f64:
    cdef C_KSlicer_Matrix1_f64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float64) if issubclass(np.float64,np.floating) else np.iinfo(np.float64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_Matrix1_f64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[double]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[double] cf
        cdef One_critical_filtration[double] inf
        inf[0] = -inf[0]
        cdef double[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.float64 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_f64(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_Matrix1_f64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_Matrix1_f64|tuple[_KSlicer_Matrix1_f64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_Matrix1_f64()
        cdef pair[C_KSlicer_Matrix1_f64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_Matrix1_f64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_Matrix1_f64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_Matrix1_f64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_Matrix1_f64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_Matrix1_f64:
        if len(self) == 0:
            return _KSlicer_Matrix1_f64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_Matrix1_f64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix1_f64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_Matrix1_f64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float64)
        cdef Line[double] line
        if direction is None:
            line = Line[double](_py21c_f64(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float64)
            line = Line[double](_py21c_f64(basepoint),_py21c_f64(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[double, double]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_Matrix1_f64._inf_value()),  dtype=np.dtype((np.float64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_Matrix1_f64._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _KSlicer_Matrix1_f64._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float64)
      cdef double[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[double]] c_basepoints
        cdef vector[pair[vector[double], vector[double]]] c_truc
        cdef vector[vector[vector[pair[double, double]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float64) if len(y)>0 else np.empty((0,2), dtype = np.float64)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_Matrix1_f64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_Matrix1_f64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_Matrix1_f64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[double],One_critical_filtration[double]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[double, ndim=1] a = _ff21cview_f64(&box.first)
        cdef cnp.ndarray[double, ndim=1] b = _ff21cview_f64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[double]] v = self.truc.get_filtration_values()
        out = _vff21cview_f64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_f64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_Matrix1_f64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_Matrix1_i32|_KSlicer_Matrix1_f64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[double]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_Matrix1_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.float64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float64
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _Slicer_Matrix0_i32:
    cdef C_Slicer_Matrix0_i32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int32) if issubclass(np.int32,np.floating) else np.iinfo(np.int32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix0_i32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.int32 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[int32_t]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_i32(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix0_i32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix0_i32|tuple[_Slicer_Matrix0_i32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix0_i32()
        cdef pair[C_Slicer_Matrix0_i32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix0_i32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix0_i32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix0_i32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix0_i32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix0_i32:
        if len(self) == 0:
            return _Slicer_Matrix0_i32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix0_i32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int32) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix0_i32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix0_i32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int32_t, int32_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix0_i32._inf_value()),  dtype=np.dtype((np.int32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int32)
      cdef int32_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int32_t]] c_basepoints
        cdef vector[pair[vector[int32_t], vector[int32_t]]] c_truc
        cdef vector[vector[vector[pair[int32_t, int32_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int32) if len(y)>0 else np.empty((0,2), dtype = np.int32)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix0_i32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix0_i32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix0_i32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int32_t],One_critical_filtration[int32_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int32_t, ndim=1] a = _ff21cview_i32(&box.first)
        cdef cnp.ndarray[int32_t, ndim=1] b = _ff21cview_i32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int32_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_i32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix0_i32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix0_i32|_Slicer_Matrix0_i32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int32_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.int32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int32
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix0_i32:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix0_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix0_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_Matrix1_i32:
    cdef C_Slicer_Matrix1_i32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int32) if issubclass(np.int32,np.floating) else np.iinfo(np.int32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix1_i32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.int32 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[int32_t]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_i32(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix1_i32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix1_i32|tuple[_Slicer_Matrix1_i32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix1_i32()
        cdef pair[C_Slicer_Matrix1_i32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix1_i32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix1_i32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix1_i32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix1_i32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix1_i32:
        if len(self) == 0:
            return _Slicer_Matrix1_i32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix1_i32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int32) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix1_i32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix1_i32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int32_t, int32_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix1_i32._inf_value()),  dtype=np.dtype((np.int32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int32)
      cdef int32_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int32_t]] c_basepoints
        cdef vector[pair[vector[int32_t], vector[int32_t]]] c_truc
        cdef vector[vector[vector[pair[int32_t, int32_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int32) if len(y)>0 else np.empty((0,2), dtype = np.int32)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix1_i32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix1_i32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix1_i32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int32_t],One_critical_filtration[int32_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int32_t, ndim=1] a = _ff21cview_i32(&box.first)
        cdef cnp.ndarray[int32_t, ndim=1] b = _ff21cview_i32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int32_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_i32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix1_i32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix1_i32|_Slicer_Matrix1_i32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int32_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix1_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.int32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int32
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix1_i32:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix1_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix1_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_Matrix0_i64:
    cdef C_Slicer_Matrix0_i64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int64) if issubclass(np.int64,np.floating) else np.iinfo(np.int64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix0_i64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.int64 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[int64_t]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_i64(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix0_i64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix0_i64|tuple[_Slicer_Matrix0_i64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix0_i64()
        cdef pair[C_Slicer_Matrix0_i64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix0_i64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix0_i64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix0_i64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix0_i64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix0_i64:
        if len(self) == 0:
            return _Slicer_Matrix0_i64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix0_i64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int64) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix0_i64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix0_i64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int64_t, int64_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix0_i64._inf_value()),  dtype=np.dtype((np.int64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int64)
      cdef int64_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int64_t]] c_basepoints
        cdef vector[pair[vector[int64_t], vector[int64_t]]] c_truc
        cdef vector[vector[vector[pair[int64_t, int64_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int64) if len(y)>0 else np.empty((0,2), dtype = np.int64)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix0_i64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix0_i64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix0_i64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int64_t],One_critical_filtration[int64_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int64_t, ndim=1] a = _ff21cview_i64(&box.first)
        cdef cnp.ndarray[int64_t, ndim=1] b = _ff21cview_i64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int64_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_i64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix0_i64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix0_i32|_Slicer_Matrix0_i64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int64_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.int64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int64
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix0_i64:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix0_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix0_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_Matrix1_i64:
    cdef C_Slicer_Matrix1_i64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int64) if issubclass(np.int64,np.floating) else np.iinfo(np.int64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix1_i64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.int64 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[int64_t]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_i64(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix1_i64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix1_i64|tuple[_Slicer_Matrix1_i64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix1_i64()
        cdef pair[C_Slicer_Matrix1_i64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix1_i64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix1_i64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix1_i64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix1_i64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix1_i64:
        if len(self) == 0:
            return _Slicer_Matrix1_i64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix1_i64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int64) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix1_i64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix1_i64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int64_t, int64_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix1_i64._inf_value()),  dtype=np.dtype((np.int64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int64)
      cdef int64_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int64_t]] c_basepoints
        cdef vector[pair[vector[int64_t], vector[int64_t]]] c_truc
        cdef vector[vector[vector[pair[int64_t, int64_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int64) if len(y)>0 else np.empty((0,2), dtype = np.int64)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix1_i64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix1_i64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix1_i64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int64_t],One_critical_filtration[int64_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int64_t, ndim=1] a = _ff21cview_i64(&box.first)
        cdef cnp.ndarray[int64_t, ndim=1] b = _ff21cview_i64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int64_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_i64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix1_i64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix1_i32|_Slicer_Matrix1_i64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int64_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix1_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.int64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int64
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix1_i64:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix1_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix1_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_Matrix0_f32:
    cdef C_Slicer_Matrix0_f32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float32) if issubclass(np.float32,np.floating) else np.iinfo(np.float32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix0_f32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.float32 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[float]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_f32(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix0_f32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix0_f32|tuple[_Slicer_Matrix0_f32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix0_f32()
        cdef pair[C_Slicer_Matrix0_f32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix0_f32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix0_f32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix0_f32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix0_f32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix0_f32:
        if len(self) == 0:
            return _Slicer_Matrix0_f32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix0_f32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix0_f32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix0_f32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float32)
        cdef Line[float] line
        if direction is None:
            line = Line[float](_py21c_f32(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float32)
            line = Line[float](_py21c_f32(basepoint),_py21c_f32(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[float, float]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix0_f32._inf_value()),  dtype=np.dtype((np.float32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix0_f32._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _Slicer_Matrix0_f32._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float32)
      cdef float[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[float]] c_basepoints
        cdef vector[pair[vector[float], vector[float]]] c_truc
        cdef vector[vector[vector[pair[float, float]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float32) if len(y)>0 else np.empty((0,2), dtype = np.float32)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix0_f32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix0_f32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix0_f32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[float],One_critical_filtration[float]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[float, ndim=1] a = _ff21cview_f32(&box.first)
        cdef cnp.ndarray[float, ndim=1] b = _ff21cview_f32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[float]] v = self.truc.get_filtration_values()
        out = _vff21cview_f32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_f32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix0_f32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix0_i32|_Slicer_Matrix0_f32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[float]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.float32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float32
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix0_f32:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix0_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix0_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_Matrix1_f32:
    cdef C_Slicer_Matrix1_f32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float32) if issubclass(np.float32,np.floating) else np.iinfo(np.float32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix1_f32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.float32 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[float]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_f32(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix1_f32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix1_f32|tuple[_Slicer_Matrix1_f32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix1_f32()
        cdef pair[C_Slicer_Matrix1_f32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix1_f32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix1_f32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix1_f32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix1_f32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix1_f32:
        if len(self) == 0:
            return _Slicer_Matrix1_f32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix1_f32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix1_f32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix1_f32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float32)
        cdef Line[float] line
        if direction is None:
            line = Line[float](_py21c_f32(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float32)
            line = Line[float](_py21c_f32(basepoint),_py21c_f32(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[float, float]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix1_f32._inf_value()),  dtype=np.dtype((np.float32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix1_f32._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _Slicer_Matrix1_f32._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float32)
      cdef float[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[float]] c_basepoints
        cdef vector[pair[vector[float], vector[float]]] c_truc
        cdef vector[vector[vector[pair[float, float]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float32) if len(y)>0 else np.empty((0,2), dtype = np.float32)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix1_f32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix1_f32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix1_f32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[float],One_critical_filtration[float]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[float, ndim=1] a = _ff21cview_f32(&box.first)
        cdef cnp.ndarray[float, ndim=1] b = _ff21cview_f32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[float]] v = self.truc.get_filtration_values()
        out = _vff21cview_f32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_f32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix1_f32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix1_i32|_Slicer_Matrix1_f32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[float]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix1_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.float32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float32
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix1_f32:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix1_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix1_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_Matrix0_f64:
    cdef C_Slicer_Matrix0_f64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float64) if issubclass(np.float64,np.floating) else np.iinfo(np.float64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix0_f64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.float64 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[double]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_f64(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix0_f64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix0_f64|tuple[_Slicer_Matrix0_f64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix0_f64()
        cdef pair[C_Slicer_Matrix0_f64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix0_f64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix0_f64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix0_f64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix0_f64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix0_f64:
        if len(self) == 0:
            return _Slicer_Matrix0_f64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix0_f64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix0_f64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix0_f64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float64)
        cdef Line[double] line
        if direction is None:
            line = Line[double](_py21c_f64(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float64)
            line = Line[double](_py21c_f64(basepoint),_py21c_f64(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[double, double]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix0_f64._inf_value()),  dtype=np.dtype((np.float64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix0_f64._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _Slicer_Matrix0_f64._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float64)
      cdef double[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[double]] c_basepoints
        cdef vector[pair[vector[double], vector[double]]] c_truc
        cdef vector[vector[vector[pair[double, double]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float64) if len(y)>0 else np.empty((0,2), dtype = np.float64)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix0_f64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix0_f64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix0_f64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[double],One_critical_filtration[double]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[double, ndim=1] a = _ff21cview_f64(&box.first)
        cdef cnp.ndarray[double, ndim=1] b = _ff21cview_f64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[double]] v = self.truc.get_filtration_values()
        out = _vff21cview_f64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_f64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix0_f64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix0_i32|_Slicer_Matrix0_f64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[double]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.float64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float64
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix0_f64:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix0_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix0_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_Matrix1_f64:
    cdef C_Slicer_Matrix1_f64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float64) if issubclass(np.float64,np.floating) else np.iinfo(np.float64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_Matrix1_f64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.float64 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[double]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_f64(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_Matrix1_f64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_Matrix1_f64|tuple[_Slicer_Matrix1_f64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_Matrix1_f64()
        cdef pair[C_Slicer_Matrix1_f64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_Matrix1_f64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_Matrix1_f64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_Matrix1_f64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_Matrix1_f64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_Matrix1_f64:
        if len(self) == 0:
            return _Slicer_Matrix1_f64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_Matrix1_f64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix1_f64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_Matrix1_f64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float64)
        cdef Line[double] line
        if direction is None:
            line = Line[double](_py21c_f64(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float64)
            line = Line[double](_py21c_f64(basepoint),_py21c_f64(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[double, double]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_Matrix1_f64._inf_value()),  dtype=np.dtype((np.float64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_Matrix1_f64._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _Slicer_Matrix1_f64._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float64)
      cdef double[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[double]] c_basepoints
        cdef vector[pair[vector[double], vector[double]]] c_truc
        cdef vector[vector[vector[pair[double, double]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float64) if len(y)>0 else np.empty((0,2), dtype = np.float64)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_Matrix1_f64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_Matrix1_f64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_Matrix1_f64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[double],One_critical_filtration[double]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[double, ndim=1] a = _ff21cview_f64(&box.first)
        cdef cnp.ndarray[double, ndim=1] b = _ff21cview_f64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[double]] v = self.truc.get_filtration_values()
        out = _vff21cview_f64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_f64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_Matrix1_f64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_Matrix1_i32|_Slicer_Matrix1_f64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[double]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_Matrix1_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.float64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float64
    @property
    def col_type(self)->str:
      return "NAIVE_VECTOR"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "Matrix"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_Matrix1_f64:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_Matrix1_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_Matrix1_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _KSlicer_GudhiCohomology0_i32:
    cdef C_KSlicer_GudhiCohomology0_i32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int32) if issubclass(np.int32,np.floating) else np.iinfo(np.int32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_GudhiCohomology0_i32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[int32_t]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[int32_t] cf
        cdef One_critical_filtration[int32_t] inf
        inf[0] = -inf[0]
        cdef int32_t[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.int32 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_i32(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_GudhiCohomology0_i32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_GudhiCohomology0_i32|tuple[_KSlicer_GudhiCohomology0_i32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_GudhiCohomology0_i32()
        cdef pair[C_KSlicer_GudhiCohomology0_i32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_GudhiCohomology0_i32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_GudhiCohomology0_i32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_GudhiCohomology0_i32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_GudhiCohomology0_i32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_GudhiCohomology0_i32:
        if len(self) == 0:
            return _KSlicer_GudhiCohomology0_i32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_GudhiCohomology0_i32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int32) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_GudhiCohomology0_i32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_GudhiCohomology0_i32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int32_t, int32_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_GudhiCohomology0_i32._inf_value()),  dtype=np.dtype((np.int32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int32)
      cdef int32_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int32_t]] c_basepoints
        cdef vector[pair[vector[int32_t], vector[int32_t]]] c_truc
        cdef vector[vector[vector[pair[int32_t, int32_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int32) if len(y)>0 else np.empty((0,2), dtype = np.int32)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_GudhiCohomology0_i32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_GudhiCohomology0_i32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_GudhiCohomology0_i32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int32_t],One_critical_filtration[int32_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int32_t, ndim=1] a = _ff21cview_i32(&box.first)
        cdef cnp.ndarray[int32_t, ndim=1] b = _ff21cview_i32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int32_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_i32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_GudhiCohomology0_i32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_GudhiCohomology0_i32|_KSlicer_GudhiCohomology0_i32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int32_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_GudhiCohomology0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.int32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int32
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "GudhiCohomology"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_GudhiCohomology0_i64:
    cdef C_KSlicer_GudhiCohomology0_i64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int64) if issubclass(np.int64,np.floating) else np.iinfo(np.int64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_GudhiCohomology0_i64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[int64_t]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[int64_t] cf
        cdef One_critical_filtration[int64_t] inf
        inf[0] = -inf[0]
        cdef int64_t[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.int64 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_i64(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_GudhiCohomology0_i64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_GudhiCohomology0_i64|tuple[_KSlicer_GudhiCohomology0_i64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_GudhiCohomology0_i64()
        cdef pair[C_KSlicer_GudhiCohomology0_i64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_GudhiCohomology0_i64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_GudhiCohomology0_i64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_GudhiCohomology0_i64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_GudhiCohomology0_i64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_GudhiCohomology0_i64:
        if len(self) == 0:
            return _KSlicer_GudhiCohomology0_i64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_GudhiCohomology0_i64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int64) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_GudhiCohomology0_i64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_GudhiCohomology0_i64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int64_t, int64_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_GudhiCohomology0_i64._inf_value()),  dtype=np.dtype((np.int64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int64)
      cdef int64_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int64_t]] c_basepoints
        cdef vector[pair[vector[int64_t], vector[int64_t]]] c_truc
        cdef vector[vector[vector[pair[int64_t, int64_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int64) if len(y)>0 else np.empty((0,2), dtype = np.int64)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_GudhiCohomology0_i64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_GudhiCohomology0_i64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_GudhiCohomology0_i64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int64_t],One_critical_filtration[int64_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int64_t, ndim=1] a = _ff21cview_i64(&box.first)
        cdef cnp.ndarray[int64_t, ndim=1] b = _ff21cview_i64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int64_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_i64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_GudhiCohomology0_i64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_GudhiCohomology0_i32|_KSlicer_GudhiCohomology0_i64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int64_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_GudhiCohomology0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.int64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int64
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "GudhiCohomology"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_GudhiCohomology0_f32:
    cdef C_KSlicer_GudhiCohomology0_f32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float32) if issubclass(np.float32,np.floating) else np.iinfo(np.float32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_GudhiCohomology0_f32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[float]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[float] cf
        cdef One_critical_filtration[float] inf
        inf[0] = -inf[0]
        cdef float[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.float32 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_f32(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_GudhiCohomology0_f32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_GudhiCohomology0_f32|tuple[_KSlicer_GudhiCohomology0_f32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_GudhiCohomology0_f32()
        cdef pair[C_KSlicer_GudhiCohomology0_f32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_GudhiCohomology0_f32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_GudhiCohomology0_f32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_GudhiCohomology0_f32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_GudhiCohomology0_f32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_GudhiCohomology0_f32:
        if len(self) == 0:
            return _KSlicer_GudhiCohomology0_f32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_GudhiCohomology0_f32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_GudhiCohomology0_f32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_GudhiCohomology0_f32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float32)
        cdef Line[float] line
        if direction is None:
            line = Line[float](_py21c_f32(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float32)
            line = Line[float](_py21c_f32(basepoint),_py21c_f32(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[float, float]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_GudhiCohomology0_f32._inf_value()),  dtype=np.dtype((np.float32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_GudhiCohomology0_f32._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _KSlicer_GudhiCohomology0_f32._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float32)
      cdef float[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[float]] c_basepoints
        cdef vector[pair[vector[float], vector[float]]] c_truc
        cdef vector[vector[vector[pair[float, float]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float32) if len(y)>0 else np.empty((0,2), dtype = np.float32)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_GudhiCohomology0_f32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_GudhiCohomology0_f32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_GudhiCohomology0_f32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[float],One_critical_filtration[float]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[float, ndim=1] a = _ff21cview_f32(&box.first)
        cdef cnp.ndarray[float, ndim=1] b = _ff21cview_f32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[float]] v = self.truc.get_filtration_values()
        out = _vff21cview_f32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_f32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_GudhiCohomology0_f32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_GudhiCohomology0_i32|_KSlicer_GudhiCohomology0_f32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[float]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_GudhiCohomology0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.float32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float32
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "GudhiCohomology"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _KSlicer_GudhiCohomology0_f64:
    cdef C_KSlicer_GudhiCohomology0_f64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float64) if issubclass(np.float64,np.floating) else np.iinfo(np.float64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_KSlicer_GudhiCohomology0_f64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - scc-like blocks 
         or 
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        if len(generator_maps)>0 and len(generator_dimensions) == 0 and len(filtration_values) == 0:
          from multipers._slicer_meta import _blocks2boundary_dimension_grades
          generator_maps, generator_dimensions, filtration_values = _blocks2boundary_dimension_grades(
                  generator_maps,
                  inplace=False,
              )
        cdef uint32_t num_generators = len(generator_maps)
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[Multi_critical_filtration[double]] c_filtration_values
        for stuff in generator_maps:
            c_generator_maps.push_back(<vector[uint32_t]>(stuff))
        cdef Multi_critical_filtration[double] cf
        cdef One_critical_filtration[double] inf
        inf[0] = -inf[0]
        cdef double[:,:] F_view 
        for F in filtration_values:
            cf.push_to_least_common_upper_bound(inf)
            F_view = np.asarray(F, dtype = np.float64 )
            for i in range(F_view.shape[0]):
                cf.add_generator(_py21c_f64(F_view[i]))
            c_filtration_values.push_back(cf)
        cdef vector[int] c_generator_dimensions = generator_dimensions
        assert num_generators == c_generator_maps.size() == c_filtration_values.size(), "Invalid input, shape do not coincide."
        self.truc = C_KSlicer_GudhiCohomology0_f64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_KSlicer_GudhiCohomology0_f64|tuple[_KSlicer_GudhiCohomology0_f64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _KSlicer_GudhiCohomology0_f64()
        cdef pair[C_KSlicer_GudhiCohomology0_f64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_KSlicer_GudhiCohomology0_f64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _KSlicer_GudhiCohomology0_f64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_KSlicer_GudhiCohomology0_f64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _KSlicer_GudhiCohomology0_f64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_KSlicer_GudhiCohomology0_f64:
        if len(self) == 0:
            return _KSlicer_GudhiCohomology0_f64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _KSlicer_GudhiCohomology0_f64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_GudhiCohomology0_f64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_KSlicer_GudhiCohomology0_f64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float64)
        cdef Line[double] line
        if direction is None:
            line = Line[double](_py21c_f64(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float64)
            line = Line[double](_py21c_f64(basepoint),_py21c_f64(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[double, double]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _KSlicer_GudhiCohomology0_f64._inf_value()),  dtype=np.dtype((np.float64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _KSlicer_GudhiCohomology0_f64._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _KSlicer_GudhiCohomology0_f64._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float64)
      cdef double[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[double]] c_basepoints
        cdef vector[pair[vector[double], vector[double]]] c_truc
        cdef vector[vector[vector[pair[double, double]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float64) if len(y)>0 else np.empty((0,2), dtype = np.float64)
            for y in x) for x in c_out)
        else:
            out = tuple(_KSlicer_GudhiCohomology0_f64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_KSlicer_GudhiCohomology0_f64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _KSlicer_GudhiCohomology0_f64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[double],One_critical_filtration[double]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[double, ndim=1] a = _ff21cview_f64(&box.first)
        cdef cnp.ndarray[double, ndim=1] b = _ff21cview_f64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[double]] v = self.truc.get_filtration_values()
        out = _vff21cview_f64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff2kcview_f64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_KSlicer_GudhiCohomology0_f64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_KSlicer_GudhiCohomology0_i32|_KSlicer_GudhiCohomology0_f64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[double]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _KSlicer_GudhiCohomology0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.float64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float64
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return True

    @property
    def pers_backend(self)->str:
      return "GudhiCohomology"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)


#------------------------------------------------------------------------------
cdef class _Slicer_GudhiCohomology0_i32:
    cdef C_Slicer_GudhiCohomology0_i32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int32) if issubclass(np.int32,np.floating) else np.iinfo(np.int32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_GudhiCohomology0_i32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.int32 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[int32_t]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_i32(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_GudhiCohomology0_i32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_GudhiCohomology0_i32|tuple[_Slicer_GudhiCohomology0_i32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_GudhiCohomology0_i32()
        cdef pair[C_Slicer_GudhiCohomology0_i32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_GudhiCohomology0_i32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_GudhiCohomology0_i32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_GudhiCohomology0_i32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_GudhiCohomology0_i32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_GudhiCohomology0_i32:
        if len(self) == 0:
            return _Slicer_GudhiCohomology0_i32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_GudhiCohomology0_i32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int32) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_GudhiCohomology0_i32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_GudhiCohomology0_i32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int32_t, int32_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_GudhiCohomology0_i32._inf_value()),  dtype=np.dtype((np.int32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int32)
      cdef int32_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int32_t]] c_basepoints
        cdef vector[pair[vector[int32_t], vector[int32_t]]] c_truc
        cdef vector[vector[vector[pair[int32_t, int32_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int32) if len(y)>0 else np.empty((0,2), dtype = np.int32)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_GudhiCohomology0_i32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_GudhiCohomology0_i32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_GudhiCohomology0_i32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int32_t],One_critical_filtration[int32_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int32_t, ndim=1] a = _ff21cview_i32(&box.first)
        cdef cnp.ndarray[int32_t, ndim=1] b = _ff21cview_i32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int32_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_i32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_GudhiCohomology0_i32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_GudhiCohomology0_i32|_Slicer_GudhiCohomology0_i32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int32_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_GudhiCohomology0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.int32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int32
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "GudhiCohomology"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_GudhiCohomology0_i32:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_GudhiCohomology0_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_GudhiCohomology0_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_GudhiCohomology0_i64:
    cdef C_Slicer_GudhiCohomology0_i64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.int64) if issubclass(np.int64,np.floating) else np.iinfo(np.int64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_GudhiCohomology0_i64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.int64 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[int64_t]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_i64(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_GudhiCohomology0_i64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_GudhiCohomology0_i64|tuple[_Slicer_GudhiCohomology0_i64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_GudhiCohomology0_i64()
        cdef pair[C_Slicer_GudhiCohomology0_i64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_GudhiCohomology0_i64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_GudhiCohomology0_i64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_GudhiCohomology0_i64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_GudhiCohomology0_i64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_GudhiCohomology0_i64:
        if len(self) == 0:
            return _Slicer_GudhiCohomology0_i64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_GudhiCohomology0_i64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.int64) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_GudhiCohomology0_i64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_GudhiCohomology0_i64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        raise NotImplementedError("There is no `int` slicing.")

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[int64_t, int64_t]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_GudhiCohomology0_i64._inf_value()),  dtype=np.dtype((np.int64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        if not self.is_squeezed:
            raise ValueError("Unsqueeze tensor, or provide a filtration grid. Cannot slice lines with integers...")
        api = api_from_tensor(self.filtration_grid[0])
        s = self.unsqueeze()
        fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
        projected_fil =_push_pts_to_line(fil, basepoint, direction)
        s.compute_persistence(projected_fil)
        bcs_idx = s.get_barcode_idx()
        bcs = tuple(
                api.stack([projected_fil[bc[:,0]], projected_fil[bc[:,1]]], axis=1) 
                if bc.size>0 
                else api.empty((0,2), dtype = self.filtration_grid[0].dtype)
                for bc in bcs_idx
            )
        if full:
            bcs = self._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.int64)
      cdef int64_t[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[int64_t]] c_basepoints
        cdef vector[pair[vector[int64_t], vector[int64_t]]] c_truc
        cdef vector[vector[vector[pair[int64_t, int64_t]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.int64) if len(y)>0 else np.empty((0,2), dtype = np.int64)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_GudhiCohomology0_i64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_GudhiCohomology0_i64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_GudhiCohomology0_i64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[int64_t],One_critical_filtration[int64_t]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[int64_t, ndim=1] a = _ff21cview_i64(&box.first)
        cdef cnp.ndarray[int64_t, ndim=1] b = _ff21cview_i64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[int64_t]] v = self.truc.get_filtration_values()
        out = _vff21cview_i64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_i64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_GudhiCohomology0_i64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_GudhiCohomology0_i32|_Slicer_GudhiCohomology0_i64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[int64_t]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_GudhiCohomology0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.int64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.int64
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "GudhiCohomology"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_GudhiCohomology0_i64:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_GudhiCohomology0_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_GudhiCohomology0_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_GudhiCohomology0_f32:
    cdef C_Slicer_GudhiCohomology0_f32 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float32) if issubclass(np.float32,np.floating) else np.iinfo(np.float32).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_GudhiCohomology0_f32*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.float32 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[float]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_f32(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_GudhiCohomology0_f32(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_GudhiCohomology0_f32|tuple[_Slicer_GudhiCohomology0_f32,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_GudhiCohomology0_f32()
        cdef pair[C_Slicer_GudhiCohomology0_f32, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_GudhiCohomology0_f32:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_GudhiCohomology0_f32()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_GudhiCohomology0_f32:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_GudhiCohomology0_f32()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_GudhiCohomology0_f32:
        if len(self) == 0:
            return _Slicer_GudhiCohomology0_f32()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_GudhiCohomology0_f32()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_GudhiCohomology0_f32._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_GudhiCohomology0_f32:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float32)
        cdef Line[float] line
        if direction is None:
            line = Line[float](_py21c_f32(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float32)
            line = Line[float](_py21c_f32(basepoint),_py21c_f32(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[float, float]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_GudhiCohomology0_f32._inf_value()),  dtype=np.dtype((np.float32,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float32) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_GudhiCohomology0_f32._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _Slicer_GudhiCohomology0_f32._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float32)
      cdef float[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[float]] c_basepoints
        cdef vector[pair[vector[float], vector[float]]] c_truc
        cdef vector[vector[vector[pair[float, float]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float32) if len(y)>0 else np.empty((0,2), dtype = np.float32)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_GudhiCohomology0_f32._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_GudhiCohomology0_f32._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_GudhiCohomology0_f32(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[float],One_critical_filtration[float]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[float, ndim=1] a = _ff21cview_f32(&box.first)
        cdef cnp.ndarray[float, ndim=1] b = _ff21cview_f32(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[float]] v = self.truc.get_filtration_values()
        out = _vff21cview_f32(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_f32(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_GudhiCohomology0_f32:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_GudhiCohomology0_i32|_Slicer_GudhiCohomology0_f32:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[float]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_GudhiCohomology0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.float32,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float32
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "GudhiCohomology"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_GudhiCohomology0_f32:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_GudhiCohomology0_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_GudhiCohomology0_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

#------------------------------------------------------------------------------
cdef class _Slicer_GudhiCohomology0_f64:
    cdef C_Slicer_GudhiCohomology0_f64 truc
    cdef public object filtration_grid
    cdef public int minpres_degree ## TODO : maybe change directly the degree in the minpres ?

    def __repr__(self):
        return f"slicer[backend={self.pers_backend},dtype={np.dtype(self.dtype).name},num_param={self.num_parameters},vineyard={self.is_vine},kcritical={self.is_kcritical},is_squeezed={self.is_squeezed},is_minpres={self.is_minpres},max_dim={self.dimension}]" 

    @property
    def is_squeezed(self)->bool:
        return self.filtration_grid is not None and len(self.filtration_grid) > 0 and len(self.filtration_grid[0]) > 0
    @property
    def is_minpres(self)->bool:
        return self.minpres_degree>=0
    @staticmethod
    def _inf_value():
        return np.asarray(np.inf,dtype=np.float64) if issubclass(np.float64,np.floating) else np.iinfo(np.float64).max

    def get_ptr(self):
        """
        Returns a pointer to the underlying C++ slicer.
        """
        return <intptr_t>(&self.truc)
    def _from_ptr(self, intptr_t slicer_ptr):
        self.truc = dereference(<C_Slicer_GudhiCohomology0_f64*>(slicer_ptr))
        return self

    def __init__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):        
        """
        Constructs a slicer from
         - generator maps (Iterable of list of ints)
         - generator dimensions (Iterable of int)
         - filtration values (Iterable of filtration values)
        """
        pass
    @cython.boundscheck(False)  
    @cython.wraparound(False)
    def __cinit__(self, generator_maps=[], generator_dimensions=[], filtration_values=[]):
        """
        Cython constructor
        """
        cdef uint32_t num_generators = len(generator_maps)
        filtration_values = np.asarray(filtration_values, dtype = np.float64 )
        assert len(filtration_values) == num_generators, f"Invalid input, shape do not coicide. Got sizes {num_generators,len(generator_dimensions),len(filtration_values)}."
        cdef vector[vector[uint32_t]] c_generator_maps
        cdef vector[One_critical_filtration[double]] c_filtration_values

        c_generator_maps.resize(num_generators)
        c_filtration_values.resize(num_generators)
        for i in range(num_generators):
            c_generator_maps[i]    = <vector[uint32_t]>(generator_maps[i])
            c_filtration_values[i] = _py21c_f64(filtration_values[i])
        cdef vector[int] c_generator_dimensions = generator_dimensions

        assert num_generators == c_generator_maps.size() == c_filtration_values.size() == c_generator_dimensions.size(), "Invalid input, shape do not coincide."
        self.truc = C_Slicer_GudhiCohomology0_f64(c_generator_maps,c_generator_dimensions, c_filtration_values)
        self.minpres_degree = -1
    
    def to_colexical(self, bool return_permutation = False)->_Slicer_GudhiCohomology0_f64|tuple[_Slicer_GudhiCohomology0_f64,np.ndarray]:
        # assert not self.is_squeezed, "Unsqueeze first, this is not implented yet for squeezed slicers"
        new_slicer = _Slicer_GudhiCohomology0_f64()
        cdef pair[C_Slicer_GudhiCohomology0_f64, vector[unsigned int]] stuff = self.truc.colexical_rearange()

        new_slicer.truc = stuff.first
        new_slicer.minpres_degree = self.minpres_degree
        new_slicer.filtration_grid = self.filtration_grid

        if return_permutation:
            return new_slicer, np.array(stuff.second, dtype=np.int32)
        return new_slicer
    def permute_generators(self, permutation)->_Slicer_GudhiCohomology0_f64:
        cdef vector[unsigned int] c_perm = permutation
        new_slicer = _Slicer_GudhiCohomology0_f64()
        new_slicer.truc = self.truc.permute(c_perm)
        new_slicer.minpres_degree = self.minpres_degree
        return new_slicer

    def copy(self)->_Slicer_GudhiCohomology0_f64:
        """
        Returns a copy of the slicer.
        """
        copy_ = _Slicer_GudhiCohomology0_f64()
        copy_.truc = self.truc
        copy_.minpres_degree = self.minpres_degree
        copy_.filtration_grid = self.filtration_grid
        return copy_
    def compute_kernel_projective_cover(self, dim:Optional[int]=None)->_Slicer_GudhiCohomology0_f64:
        if len(self) == 0:
            return _Slicer_GudhiCohomology0_f64()
        if dim is None:
            dim = self.truc.get_dimension(0)
        out = _Slicer_GudhiCohomology0_f64()
        out.truc = self.truc.projective_cover_kernel(dim)
        out.filtration_grid = self.filtration_grid
        return out

    def get_barcode_idx(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        return tuple(np.asarray(x) if len(x) else np.empty((0,2), dtype=int)for x in self.truc.get_barcode_idx())
    def get_barcode(self, bool keep_inf = False):
        """
        Returns the current barcode.
        """
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_GudhiCohomology0_f64._threshold_bcs(self.truc.get_barcode())
        return bcs
    def push_to_line(self, basepoint, direction=None)->_Slicer_GudhiCohomology0_f64:
        """
        Pushes the current slicer to the line defined by a basepoint and an optional direction.
        If the direction is not provided, it is assumed to be diagonal.
        """
        basepoint = np.asarray(basepoint, dtype = np.float64)
        cdef Line[double] line
        if direction is None:
            line = Line[double](_py21c_f64(basepoint))
        else:
            direction = np.asarray(direction, dtype = np.float64)
            line = Line[double](_py21c_f64(basepoint),_py21c_f64(direction))
        self.truc.push_to(line)
        return self

    @staticmethod
    cdef _threshold_bcs(vector[vector[pair[double, double]]] bcs):
        return tuple(np.fromiter((a for a in stuff if a.first < _Slicer_GudhiCohomology0_f64._inf_value()),  dtype=np.dtype((np.float64,2))) for stuff in bcs)
    @staticmethod
    def _bc_to_full(bcs, basepoint, direction=None):
        # i, (b sv d), coords 
        basepoint = basepoint[None,None,:]
        direction = 1 if direction is None else direction[None,None,:]
        return tuple(bc[:,:,None]*direction + basepoint for bc in bcs)

    def persistence_on_line(self,basepoint,direction=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Computes the persistence on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        
        Warning: This is not parallelizable. Use `persitence_on_lines`.
        """
        self.push_to_line(basepoint,direction)
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        if keep_inf:
            bcs = tuple(np.asarray(stuff, dtype = np.float64) for stuff in self.truc.get_barcode())
        else:
            bcs = _Slicer_GudhiCohomology0_f64._threshold_bcs(self.truc.get_barcode())

        if full:
            bcs = _Slicer_GudhiCohomology0_f64._bc_to_full(bcs, basepoint, direction)
        return bcs

    def _custom_persistences_idx(self, filtration_array, bool ignore_inf=True):
      filtration_array = np.asarray(filtration_array, dtype= np.float64)
      cdef double[:,:] arr_view = filtration_array
      cdef int size = arr_view.shape[0]
      if arr_view.shape[1] != self.truc.num_generators():
          raise ValueError(f"Got filtration array of shape {filtration_array.shape=} / {arr_view.shape=}. Was expecting (-1, {len(self)=})")

      return tuple(tuple(np.array(bc_idx_degree, dtype=int) for bc_idx_degree in bc_idx) for bc_idx in self.truc.custom_persistences(&arr_view[0,0], size, ignore_inf))

    def persistence_on_lines(self, basepoints=None, directions=None, bool keep_inf=True, bool full=False, bool ignore_infinite_filtration_values = True):
        """
        Same as `persistence_on_line`, but with vineyards operation between
        lines if `self.is_vine`, and in parallel otherwise.
        """
        cdef vector[vector[double]] c_basepoints
        cdef vector[pair[vector[double], vector[double]]] c_truc
        cdef vector[vector[vector[pair[double, double]]]] c_out 
        if directions is None:
            c_basepoints = basepoints
            with nogil:
                c_out = self.truc.persistence_on_lines(c_basepoints, ignore_infinite_filtration_values)
        else:
            c_truc = zip(basepoints,directions)
            with nogil:
                c_out = self.truc.persistence_on_lines(c_truc, ignore_infinite_filtration_values)
        cdef int num_bc = c_basepoints.size()
        
        if keep_inf:
            out = tuple(tuple(
                np.asarray(y, dtype = np.float64) if len(y)>0 else np.empty((0,2), dtype = np.float64)
            for y in x) for x in c_out)
        else:
            out = tuple(_Slicer_GudhiCohomology0_f64._threshold_bcs(x) for x in c_out)

        if full:
            _dirs = [None]*len(basepoints) if directions is None else directions 
            out = tuple(_Slicer_GudhiCohomology0_f64._bc_to_full(bcs, bp, dir) for bcs, bp, dir in zip(out,basepoints,_dirs))
        return out


    def __getstate__(self):
        return (
            self.get_boundaries(),
            self.get_dimensions(),
            self.get_filtrations(),
            self.filtration_grid,
            self.minpres_degree,
        )
    def __setstate__(self, tuple dump):
        B, D, F, filtration_grid, minpres_degree = dump
        copy = _Slicer_GudhiCohomology0_f64(B,D,F)
        self.truc = copy.truc
        self.minpres_degree= minpres_degree
        self.filtration_grid=filtration_grid

    def __eq__(self, other):
        ## True if they rpz the same complex (no reordering allowed yet), 
        # i.e., ignores minpres, squeeze
        if other.is_squeezed:
            return self == other.unsqueeze()
        if self.is_squeezed:
            return self.unsqueeze() == other
        return (
                np.array_equal(self.get_dimensions(), other.get_dimensions())
                and
                self.get_boundaries() == other.get_boundaries() 
                and 
                ## Kcritical are sorted + filtrationvalues is a dump. 
                # for non kcritical there is an unnecessary copy though
                np.array_equal(self.get_filtrations_values(), other.get_filtrations_values())
            )



    def compute_persistence(self,one_filtration=None, bool ignore_infinite_filtration_values = True)->tuple:
        """
        Computes the current persistence, or the persistence
        given by the filtration one_filtration (num_generators,).
        """
        if one_filtration is not None:
            api = api_from_tensor(one_filtration)
            one_filtration=api.astensor(one_filtration)
            # self.truc.set_one_filtration(one_filtration)
            # s = self.unsqueeze()
            # fil = evaluate_in_grid(np.asarray(self.get_filtrations()), self.filtration_grid)
            # projected_fil =_push_pts_to_line(fil, basepoint, direction)
            if one_filtration.ndim > 2 or one_filtration.ndim == 0:
                raise ValueError(f"Expected a filtration shape of the form ((num_1_param), num_generators). Got {one_filtration.shape=}")
            squeeze = False
            if one_filtration.ndim == 1:
                one_filtration = one_filtration[None]
                squeeze = True

            bcs = self._custom_persistences_idx(api.asnumpy(one_filtration),ignore_infinite_filtration_values)
            
            bcs = tuple(tuple(
                    api.stack([F[bc[:,0]], F[bc[:,1]]], axis=1) 
                    if bc.size>0 
                    else api.empty((0,2), dtype = F.dtype)
                    for bc in bcs_idx
                )
                for bcs_idx,F in zip(bcs,one_filtration)
            )
            return bcs[0] if squeeze else bcs

        # TODO: Later
        # if len(degrees)>0:
        #     self.truc.compute_persistence(degrees)
        # else:
        #     self.truc.compute_persistence()
        self.truc.compute_persistence(ignore_infinite_filtration_values)
        # return self
        return self.get_barcode()
    def get_barcode(self):
        """
        Returns the barcode of the current 1d-persistence.
        """
        return tuple(np.asarray(bc) for bc in self.truc.get_barcode())
    def sliced_filtration(self,basepoint, direction=None):
        """
        Computes the filtration on a line L defined by 
         - a basepoint (num_parameters,) array
         - an optional direction (num_parameters,) array
        """
        self.push_to_line(basepoint,direction)
        return np.asarray(self.truc.get_one_filtration())
    def __len__(self):
        return self.truc.num_generators()
    @property
    def num_generators(self):
        return self.truc.num_generators()
    @property
    def num_parameters(self):
        return self.truc.num_parameters()
    @property
    def info(self):
        print(self.truc.to_str().decode())
    def filtration_bounds(self) -> np.ndarray:
        """
        Computes the bounding box of the current multifiltration.
        """
        cdef pair[One_critical_filtration[double],One_critical_filtration[double]] box = self.truc.get_bounding_box()
        cdef cnp.ndarray[double, ndim=1] a = _ff21cview_f64(&box.first)
        cdef cnp.ndarray[double, ndim=1] b = _ff21cview_f64(&box.second)
        return np.asarray([a,b])
    def get_filtrations_values(self)->np.ndarray:
        """
        Returns the current filtration values of the slicer.
        """
        cdef vector[One_critical_filtration[double]] v = self.truc.get_filtration_values()
        out = _vff21cview_f64(v, copy=True, duplicate=self.num_parameters)
        return np.asarray(out)
    def get_filtration_grid(self,grid_strategy:str="exact", **infer_grid_kwargs):
        return compute_grid(
                self.get_filtrations_values().T,
                strategy=grid_strategy,
                **infer_grid_kwargs,
            )
    def get_filtrations(self):
        """
        Returns a view of the filtration values, as a list of numpy arrays.
        """
        return _vff21cview_f64(self.truc.get_filtrations(), copy=False, duplicate=self.num_parameters)

    def get_dimensions(self)-> np.ndarray:
        """
        Returns the ordered dimensions of the generators.
        """
        return np.asarray(self.truc.get_dimensions())
    @property 
    def dimension(self)-> int:
        """
        Returns the maximum dimension of the complex.
        """
        return self.get_dimensions()[-1] if len(self)>0 else -np.inf
    def prune_above_dimension(self,int max_dimension)->_Slicer_GudhiCohomology0_f64:
        """
        Prunes the generators above a given dimension.
        """
        self.truc.prune_above_dimension(max_dimension)
        return self
    def get_boundaries(self)->tuple[tuple]:
        """
        Returns the boundaries of the generators.
        """
        return tuple(tuple(b) for b in self.truc.get_boundaries())
    def grid_squeeze(
            self, 
            filtration_grid=None,
            strategy="exact",
            resolution:Optional[int]=None,
            bool coordinates=True,
            bool inplace = False,
            grid_strategy=None
        )->_Slicer_GudhiCohomology0_i32|_Slicer_GudhiCohomology0_f64:
        """
        Coarsen the filtration values on a grid. This is necessary to compute some invariants.

        If the filtration grid is not given, it is infered from filtration values,
        using the :func:`multipers.grids.compute_grid` function, whose args are
          - grid_strategy:str see `multipers.grids.available_strategies`. Defaults to exact.
          - resolution:int if strategy is not exact.

         - inplace:bool if true, does the operation inplace, i.e., doesn't return a copy.
        """
        if grid_strategy is not None:
            warn("`grid_strategy` is deprecated, use `strategy` instead.",DeprecationWarning)
            strategy=grid_strategy

        if self.is_squeezed:
            warn("(copy warning) Squeezing an already squeezed slicer.")
            temp = self.unsqueeze()
            subgrid = compute_grid(self.filtration_grid, strategy=strategy, resolution=resolution)
            return temp.grid_squeeze(subgrid, coordinates=coordinates, inplace=inplace)

        if filtration_grid is None:
            filtration_grid = compute_grid(
                    self.get_filtrations_values().T,
                    strategy=strategy,
                    resolution=resolution)
        cdef vector[vector[double]] grid = filtration_grid
        if inplace or not coordinates:
            self.truc.coarsen_on_grid_inplace(grid, coordinates)
            if coordinates:
                self.filtration_grid = filtration_grid
        else:
          out = _Slicer_GudhiCohomology0_i32()
          out.truc = self.truc.coarsen_on_grid(grid)
          if coordinates:
              out.filtration_grid = sanitize_grid(filtration_grid)
          out.minpres_degree = self.minpres_degree
          return out
        return self
    def _clean_filtration_grid(self):
        """
        Removes the values in filtration_grid that are not linked to any splx.
        """
        if not self.is_squeezed:
            raise ValueError("No grid to clean.")
        F = self.filtration_grid
        self.filtration_grid=None
        cleaned_coordinates = compute_grid(self)
        new_slicer = self.grid_squeeze(cleaned_coordinates)

        self._from_ptr(new_slicer.get_ptr())
        self.filtration_grid = tuple(f[g] for f,g in zip(F,cleaned_coordinates))
        return self

    def minpres(self,
        int degree=-1, 
        list[int] degrees=[],
        str backend:Literal["mpfree", "2pac"]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=False, 
        id :Optional[str] = None,
        dtype = np.float64,
        **minpres_kwargs
        )->Slicer_type:
        """
        Computes the minimal presentation of the slicer, and returns it as a new slicer.
        See :func:`multipers.slicer.minimal_presentation`.
        """
        new_slicer = minimal_presentation(self, degree=degree, degrees=degrees, backend=backend, slicer_backend=slicer_backend, dtype=dtype, vineyard=vineyard, id=id, **minpres_kwargs)
        return new_slicer

    @property
    def dtype(self)->type:
      return np.float64
    @property
    def col_type(self)->str:
      return "INTRUSIVE_SET"
    @property
    def is_vine(self)->bool:
      return False
    @property
    def is_kcritical(self)->bool:
      return False

    @property
    def pers_backend(self)->str:
      return "GudhiCohomology"


    def to_scc(
            self,
            path:os.PathLike,
            int num_parameters = -1,
            int degree = -1,
            bool rivet_compatible = False,
            bool ignore_last_generators = False,
            bool strip_comments = False,
            bool reverse = False,
            bool unsqueeze = True,
        ):
        """
        Writes current slicer to a file in scc format.
        """
        if degree == -1 and not rivet_compatible:
            degree = 1
        cdef string c_path = path.encode(encoding="utf-8")
        if self.is_squeezed and unsqueeze:
            kwargs = locals()
            kwargs.pop("self",0)
            kwargs.pop("c_path",0)
            self.unsqueeze().to_scc(**kwargs)
            return
        with nogil:
          self.truc.write_to_scc_file(c_path, num_parameters, degree, rivet_compatible, ignore_last_generators, strip_comments, reverse)

    def _build_from_scc_file(self, path:os.PathLike, bool rivet_compatible = False, bool reverse = False, int shift_dimension = 0)->_Slicer_GudhiCohomology0_f64:
        """
        Builds the slicer from the given scc file. Should be empty before, otherwise will be completely overwritten.
        """
        cdef string c_path = path.encode(encoding="utf-8")
        with nogil:
          self.truc.build_from_scc_file(c_path, rivet_compatible, reverse, shift_dimension)
        return self

    def unsqueeze(self, grid=None)->_Slicer_GudhiCohomology0_f64:
        from multipers.grids import evaluate_in_grid, sanitize_grid
        from multipers import Slicer
        grid = self.filtration_grid if grid is None else grid
        grid = sanitize_grid(grid, numpyfy=True)
        new_filtrations = evaluate_in_grid(np.asarray(self.get_filtrations(), dtype=np.int32), grid)
        new_slicer = _Slicer_GudhiCohomology0_f64(
            self.get_boundaries(),
            self.get_dimensions(),
            new_filtrations,
        )
        new_slicer.minpres_degree=self.minpres_degree
        return new_slicer

from libcpp.vector cimport vector
from libcpp.set cimport set as cset
cdef extern from "gudhi/cubical_to_boundary.h" namespace "":
  void _to_boundary(const vector[unsigned int]&, vector[vector[unsigned int]]&, vector[int]&) except + nogil
  void get_vertices(unsigned int, cset[unsigned int]&, const vector[vector[unsigned int]]&) nogil

def _from_bitmapi32(image, **slicer_kwargs):
    from multipers import Slicer
    dtype = slicer_kwargs.get("dtype", image.dtype)
    slicer_kwargs["dtype"] = dtype
    if image.dtype  !=  dtype:
        raise ValueError(f"Invalid type matching. Got {dtype=} and {image.dtype=}")
    _Slicer = Slicer(return_type_only=True, **slicer_kwargs)
    cdef vector[unsigned int] img_shape = image.shape[:-1]
    cdef unsigned int num_parameters = image.shape[-1]
    cdef vector[vector[unsigned int]] gen_maps
    cdef vector[int] gen_dims 

    with nogil:
        _to_boundary(img_shape,gen_maps, gen_dims)

    cdef cset[unsigned int] vertices

    cdef unsigned int num_gens = gen_dims.size()
    filtration_values = np.zeros(shape=(num_gens, num_parameters), dtype = np.int32) - _Slicer._inf_value()
    cdef int32_t[:,:] F = filtration_values
    cdef int32_t[:,:] c_img = image.reshape(-1,num_parameters)
    with nogil:
        for i in range(num_gens):
            # with gil:
            #     print(f"idx {i}:", end="")
            vertices.clear()
            get_vertices(i,vertices,gen_maps)

            # with gil:
            #     print(f"v = {vertices}:", end="")
            for k in vertices:
                for j in range(num_parameters):
                    F[i,j] =  max(F[i,j], c_img[k,j])

            # with gil:
            #     print(f"F = {np.asarray(F[i])}")
    slicer = _Slicer(gen_maps, gen_dims, filtration_values)
    return slicer
def _from_bitmapf64(image, **slicer_kwargs):
    from multipers import Slicer
    dtype = slicer_kwargs.get("dtype", image.dtype)
    slicer_kwargs["dtype"] = dtype
    if image.dtype  !=  dtype:
        raise ValueError(f"Invalid type matching. Got {dtype=} and {image.dtype=}")
    _Slicer = Slicer(return_type_only=True, **slicer_kwargs)
    cdef vector[unsigned int] img_shape = image.shape[:-1]
    cdef unsigned int num_parameters = image.shape[-1]
    cdef vector[vector[unsigned int]] gen_maps
    cdef vector[int] gen_dims 

    with nogil:
        _to_boundary(img_shape,gen_maps, gen_dims)

    cdef cset[unsigned int] vertices

    cdef unsigned int num_gens = gen_dims.size()
    filtration_values = np.zeros(shape=(num_gens, num_parameters), dtype = np.float64) - _Slicer._inf_value()
    cdef double[:,:] F = filtration_values
    cdef double[:,:] c_img = image.reshape(-1,num_parameters)
    with nogil:
        for i in range(num_gens):
            # with gil:
            #     print(f"idx {i}:", end="")
            vertices.clear()
            get_vertices(i,vertices,gen_maps)

            # with gil:
            #     print(f"v = {vertices}:", end="")
            for k in vertices:
                for j in range(num_parameters):
                    F[i,j] =  max(F[i,j], c_img[k,j])

            # with gil:
            #     print(f"F = {np.asarray(F[i])}")
    slicer = _Slicer(gen_maps, gen_dims, filtration_values)
    return slicer
def _from_bitmapf32(image, **slicer_kwargs):
    from multipers import Slicer
    dtype = slicer_kwargs.get("dtype", image.dtype)
    slicer_kwargs["dtype"] = dtype
    if image.dtype  !=  dtype:
        raise ValueError(f"Invalid type matching. Got {dtype=} and {image.dtype=}")
    _Slicer = Slicer(return_type_only=True, **slicer_kwargs)
    cdef vector[unsigned int] img_shape = image.shape[:-1]
    cdef unsigned int num_parameters = image.shape[-1]
    cdef vector[vector[unsigned int]] gen_maps
    cdef vector[int] gen_dims 

    with nogil:
        _to_boundary(img_shape,gen_maps, gen_dims)

    cdef cset[unsigned int] vertices

    cdef unsigned int num_gens = gen_dims.size()
    filtration_values = np.zeros(shape=(num_gens, num_parameters), dtype = np.float32) - _Slicer._inf_value()
    cdef float[:,:] F = filtration_values
    cdef float[:,:] c_img = image.reshape(-1,num_parameters)
    with nogil:
        for i in range(num_gens):
            # with gil:
            #     print(f"idx {i}:", end="")
            vertices.clear()
            get_vertices(i,vertices,gen_maps)

            # with gil:
            #     print(f"v = {vertices}:", end="")
            for k in vertices:
                for j in range(num_parameters):
                    F[i,j] =  max(F[i,j], c_img[k,j])

            # with gil:
            #     print(f"F = {np.asarray(F[i])}")
    slicer = _Slicer(gen_maps, gen_dims, filtration_values)
    return slicer
def _from_bitmapi64(image, **slicer_kwargs):
    from multipers import Slicer
    dtype = slicer_kwargs.get("dtype", image.dtype)
    slicer_kwargs["dtype"] = dtype
    if image.dtype  !=  dtype:
        raise ValueError(f"Invalid type matching. Got {dtype=} and {image.dtype=}")
    _Slicer = Slicer(return_type_only=True, **slicer_kwargs)
    cdef vector[unsigned int] img_shape = image.shape[:-1]
    cdef unsigned int num_parameters = image.shape[-1]
    cdef vector[vector[unsigned int]] gen_maps
    cdef vector[int] gen_dims 

    with nogil:
        _to_boundary(img_shape,gen_maps, gen_dims)

    cdef cset[unsigned int] vertices

    cdef unsigned int num_gens = gen_dims.size()
    filtration_values = np.zeros(shape=(num_gens, num_parameters), dtype = np.int64) - _Slicer._inf_value()
    cdef int64_t[:,:] F = filtration_values
    cdef int64_t[:,:] c_img = image.reshape(-1,num_parameters)
    with nogil:
        for i in range(num_gens):
            # with gil:
            #     print(f"idx {i}:", end="")
            vertices.clear()
            get_vertices(i,vertices,gen_maps)

            # with gil:
            #     print(f"v = {vertices}:", end="")
            for k in vertices:
                for j in range(num_parameters):
                    F[i,j] =  max(F[i,j], c_img[k,j])

            # with gil:
            #     print(f"F = {np.asarray(F[i])}")
    slicer = _Slicer(gen_maps, gen_dims, filtration_values)
    return slicer

def from_bitmap(img, **kwargs):
    img = np.asarray(img)
    if img.dtype == np.int32:
        return _from_bitmapi32(img, **kwargs)
    if img.dtype == np.float64:
        return _from_bitmapf64(img, **kwargs)
    if img.dtype == np.float32:
        return _from_bitmapf32(img, **kwargs)
    if img.dtype == np.int64:
        return _from_bitmapi64(img, **kwargs)
    raise ValueError(f"Invalid dtype. Got {img.dtype=}, was expecting {available_dtype=}.")

def from_function_delaunay(
    points,
    grades,
    int degree=-1,
    backend:Optional[_valid_pers_backend]=None,
    vineyard=None, # TODO : Optional[bool] when cython fixes it
    dtype=np.float64,
    bool verbose = False,
    bool clear = True,
):
    """
    Given points in $\mathbb R^n$ and function grades, compute the function-delaunay
    bifiltration as a in an scc format, and converts it into a slicer.

    points : (num_pts, n) float array
    grades : (num_pts,) float array
    degree (opt) : if given, computes a minimal presentation of this homological degree first
    backend : slicer backend, e.g. "matrix", "clement"
    vineyard : bool, use a vineyard-compatible backend
    """
    from multipers.io import _check_available, function_delaunay_presentation_to_slicer
    s = multipers.Slicer(None, backend=backend, vineyard=vineyard, dtype=dtype)
    assert _check_available("function_delaunay"), f"Could not find function_delaunay"
    function_delaunay_presentation_to_slicer(s, points, grades, degree=degree, verbose=verbose,clear=clear)
    if degree >= 0:
        s.minpres_degree = degree
    return s

def slicer2blocks(slicer, int degree = -1, bool reverse=True):
    """
    Convert any slicer to the block format a.k.a. scc format for python
    """
    dims = slicer.get_dimensions()
    num_empty_blocks_to_add = 1 if degree == -1 else dims.min()-degree +1
    _,counts = np.unique(dims, return_counts=True, )
    indices = np.concatenate([[0],counts], dtype=np.int32).cumsum()
    filtration_values = slicer.get_filtrations()
    filtration_values = [filtration_values[indices[i]:indices[i+1]] for i in range(len(indices)-1)]
    boundaries = slicer.get_boundaries()
    boundaries = [boundaries[indices[i]:indices[i+1]] for i in range(len(indices)-1)]
    shift = np.concatenate([[0], indices], dtype=np.int32)
    boundaries = [tuple(np.asarray(x-s, dtype=np.int32) for x in block)  for s,block in zip(shift,boundaries)]
    blocks = [tuple((f,tuple(b))) for b,f in zip(boundaries,filtration_values)]
    blocks = ([(np.empty((0,)),[])]*num_empty_blocks_to_add) + blocks
    if reverse:
        blocks.reverse()
    return blocks

def minimal_presentation(
        slicer,
        int degree = -1, 
        degrees:Iterable[int]=[],
        str backend:Literal["mpfree", "2pac", ""]="mpfree", 
        str slicer_backend:Literal["matrix","clement","graph"]="matrix",
        bool vineyard=True, 
        id :Optional[str] =None,
        dtype:type|_valid_dtypes=None,
        int n_jobs = -1,
        bool force=False,
        bool auto_clean = True,
        **minpres_kwargs
        ):
    """
    Computes a minimal presentation of the multifiltered complex given by the slicer,
    and returns it as a slicer.
    Backends differents than `mpfree` are unstable.
    """
    from multipers.io import _init_external_softwares, input_path, scc_reduce_from_str_to_slicer
    if is_slicer(slicer) and slicer.is_minpres and not force:
        from warnings import warn
        warn(f"(unnecessary computation) The slicer seems to be already reduced, from homology of degree {slicer.minpres_degree}.")
        return slicer
    _init_external_softwares(requires=[backend])
    if len(degrees)>0:
        def todo(int degree):
            return minimal_presentation(slicer, degree=degree, backend=backend, slicer_backend=slicer_backend, vineyard=vineyard, id=id, **minpres_kwargs)
        return tuple(
          Parallel(n_jobs=n_jobs, backend="threading")(delayed(todo)(d) for d in degrees)
        )
        # return tuple(minimal_presentation(slicer, degree=d, backend=backend, slicer_backend=slicer_backend, vineyard=vineyard, id=id, **minpres_kwargs) for d in degrees)
    assert degree>=0, f"Degree not provided."
    if not np.any(slicer.get_dimensions() == degree):
        return type(slicer)()
    if id is None:
        id = str(threading.get_native_id())
    if dtype is None:
        dtype = slicer.dtype
    dimension = slicer.dimension - degree # latest  = L-1, which is empty, -1 for degree 0, -2 for degree 1 etc.
    slicer.to_scc(path=input_path+id, strip_comments=True, degree=degree-1, unsqueeze = False)
    new_slicer = multipers.Slicer(None,backend=slicer_backend, vineyard=vineyard, dtype=dtype)
    if backend=="mpfree":
        shift_dimension=degree-1
    else:
        shift_dimension=degree
    scc_reduce_from_str_to_slicer(path=input_path+id, slicer=new_slicer, dimension=dimension, backend=backend, shift_dimension=shift_dimension, **minpres_kwargs)

    new_slicer.minpres_degree = degree
    new_slicer.filtration_grid = slicer.filtration_grid if slicer.is_squeezed else None
    if new_slicer.is_squeezed and auto_clean:
        new_slicer = new_slicer._clean_filtration_grid()
    return new_slicer


def to_simplextree(s:Slicer_type, max_dim:int=-1) -> SimplexTreeMulti_type:
    """
    Turns a --simplicial-- slicer into a simplextree.

    Warning: Won't work for non-simplicial complexes, 
    i.e., complexes $K$ not satisfying 
    $\forall \sigma \in K,\, \mathrm{dim}(\sigma) = |\partial \sigma|-1$
    """
    dims = s.get_dimensions()
    assert np.all(dims[:-1] <= dims[1:]), "Dims is not sorted."
    idx = np.searchsorted(dims, np.unique(dims))
    idx = np.concatenate([idx, [dims.shape[0]]])
    if max_dim>=0:
        idx = idx[:max_dim+2]

    cdef vector[vector[int]] boundaries_ = s.get_boundaries()
    cdef int a
    cdef int b
    if len(idx)>2:
        a = idx[2]
        b = idx[-1]
        for i in range(a, b):
            boundaries_[i] = np.unique(np.concatenate([boundaries_[k] for k in boundaries_[i]]))
    cdef int num_dims = len(idx)-1
    boundaries = [np.asarray(boundaries_[idx[i]:idx[i+1]], dtype=np.int32).T for i in range(num_dims)]
    boundaries[0] = np.arange(boundaries[0].shape[1])[None,:]
    filtrations = s.get_filtrations()
    num_parameters  = s.num_parameters
    filtrations=tuple(filtrations[idx[i]:idx[i+1]] for i in range(num_dims)) # TODO : optimize ?
    st = SimplexTreeMulti(num_parameters = num_parameters, dtype = s.dtype, kcritical=s.is_kcritical)
    for dim in range(num_dims):
        if s.is_kcritical: ## TODO : this may be very slow
          # for b,f in zip(boundaries[i].T,filtrations[i]):
          #   for g in np.asarray(f):
          #       st.insert(np.asarray(b, dtype = np.int32),np.asarray(g, dtype=s.dtype))
          for j in range(boundaries[i].shape[1]):
              splx = boundaries[i][:,j]
              for k in range(filtrations[i][j]):
                st.insert(splx, np.asarray(filtrations[i][j][k], dtype = s.dtype))
        else:
          st.insert_batch(np.asarray(boundaries[dim], dtype= np.int32),np.asarray(filtrations[dim], dtype=s.dtype))
    return st


def _is_slicer(object input)->bool:
    """
    Checks if the input is a slicer. Equivalent (but faster) to `isinstance(input, multipers.slicer.Slicer_type)`.
    """
    return (False 
        or isinstance(input, _KSlicer_Matrix0_vine_i32)
        or isinstance(input, _KSlicer_Matrix1_vine_i32)
        or isinstance(input, _KSlicer_Matrix0_vine_i64)
        or isinstance(input, _KSlicer_Matrix1_vine_i64)
        or isinstance(input, _KSlicer_Matrix0_vine_f32)
        or isinstance(input, _KSlicer_Matrix1_vine_f32)
        or isinstance(input, _KSlicer_Matrix0_vine_f64)
        or isinstance(input, _KSlicer_Matrix1_vine_f64)
        or isinstance(input, _Slicer_Matrix0_vine_i32)
        or isinstance(input, _Slicer_Matrix1_vine_i32)
        or isinstance(input, _Slicer_Matrix0_vine_i64)
        or isinstance(input, _Slicer_Matrix1_vine_i64)
        or isinstance(input, _Slicer_Matrix0_vine_f32)
        or isinstance(input, _Slicer_Matrix1_vine_f32)
        or isinstance(input, _Slicer_Matrix0_vine_f64)
        or isinstance(input, _Slicer_Matrix1_vine_f64)
        or isinstance(input, _KSlicer_Matrix0_i32)
        or isinstance(input, _KSlicer_Matrix1_i32)
        or isinstance(input, _KSlicer_Matrix0_i64)
        or isinstance(input, _KSlicer_Matrix1_i64)
        or isinstance(input, _KSlicer_Matrix0_f32)
        or isinstance(input, _KSlicer_Matrix1_f32)
        or isinstance(input, _KSlicer_Matrix0_f64)
        or isinstance(input, _KSlicer_Matrix1_f64)
        or isinstance(input, _Slicer_Matrix0_i32)
        or isinstance(input, _Slicer_Matrix1_i32)
        or isinstance(input, _Slicer_Matrix0_i64)
        or isinstance(input, _Slicer_Matrix1_i64)
        or isinstance(input, _Slicer_Matrix0_f32)
        or isinstance(input, _Slicer_Matrix1_f32)
        or isinstance(input, _Slicer_Matrix0_f64)
        or isinstance(input, _Slicer_Matrix1_f64)
        or isinstance(input, _KSlicer_GudhiCohomology0_i32)
        or isinstance(input, _KSlicer_GudhiCohomology0_i64)
        or isinstance(input, _KSlicer_GudhiCohomology0_f32)
        or isinstance(input, _KSlicer_GudhiCohomology0_f64)
        or isinstance(input, _Slicer_GudhiCohomology0_i32)
        or isinstance(input, _Slicer_GudhiCohomology0_i64)
        or isinstance(input, _Slicer_GudhiCohomology0_f32)
        or isinstance(input, _Slicer_GudhiCohomology0_f64)
      )
def is_slicer(input, bool allow_minpres=True)->bool:
    if _is_slicer(input):
        return True
    if allow_minpres and isinstance(input, list) or isinstance(input, tuple):
        if len(input)>0 and all((_is_slicer(s) and s.is_minpres for s in input)):
            return True
    return False


def to_blocks(input):
    """
    Converts input to blocks, if possible.
    """
    if is_slicer(input):
      return slicer2blocks(input)
    if isinstance(input, list) or isinstance(input, tuple):
      return input
    from multipers.simplex_tree_multi import is_simplextree_multi
    if is_simplextree_multi(input):
      return input._to_scc()
    if isinstance(input, str) or isinstance(input, os.PathLike):
      from multipers.io import scc_parser
      return scc_parser(input)
    raise ValueError("Input cannot be converted to blocks.")


def get_matrix_slicer(bool is_vineyard, bool is_k_critical, dtype:type|_valid_dtypes, str col, str pers_backend):
  """
  Given various parameters, returns the specific slicer type associated with them.
  """
  if False:
      pass
  elif is_vineyard == True and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.int32) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix0_vine_i32
  elif is_vineyard == True and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.int32) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix1_vine_i32
  elif is_vineyard == True and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.int64) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix0_vine_i64
  elif is_vineyard == True and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.int64) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix1_vine_i64
  elif is_vineyard == True and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.float32) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix0_vine_f32
  elif is_vineyard == True and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.float32) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix1_vine_f32
  elif is_vineyard == True and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.float64) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix0_vine_f64
  elif is_vineyard == True and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.float64) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix1_vine_f64
  elif is_vineyard == True and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.int32) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix0_vine_i32
  elif is_vineyard == True and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.int32) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix1_vine_i32
  elif is_vineyard == True and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.int64) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix0_vine_i64
  elif is_vineyard == True and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.int64) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix1_vine_i64
  elif is_vineyard == True and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.float32) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix0_vine_f32
  elif is_vineyard == True and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.float32) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix1_vine_f32
  elif is_vineyard == True and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.float64) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix0_vine_f64
  elif is_vineyard == True and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.float64) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix1_vine_f64
  elif is_vineyard == False and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.int32) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix0_i32
  elif is_vineyard == False and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.int32) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix1_i32
  elif is_vineyard == False and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.int64) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix0_i64
  elif is_vineyard == False and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.int64) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix1_i64
  elif is_vineyard == False and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.float32) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix0_f32
  elif is_vineyard == False and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.float32) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix1_f32
  elif is_vineyard == False and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.float64) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix0_f64
  elif is_vineyard == False and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.float64) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _KSlicer_Matrix1_f64
  elif is_vineyard == False and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.int32) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix0_i32
  elif is_vineyard == False and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.int32) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix1_i32
  elif is_vineyard == False and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.int64) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix0_i64
  elif is_vineyard == False and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.int64) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix1_i64
  elif is_vineyard == False and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.float32) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix0_f32
  elif is_vineyard == False and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.float32) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix1_f32
  elif is_vineyard == False and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.float64) and col.lower() == "INTRUSIVE_SET".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix0_f64
  elif is_vineyard == False and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.float64) and col.lower() == "NAIVE_VECTOR".lower() and "Matrix".lower() == pers_backend.lower():
    return _Slicer_Matrix1_f64
  elif is_vineyard == False and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.int32) and col.lower() == "INTRUSIVE_SET".lower() and "GudhiCohomology".lower() == pers_backend.lower():
    return _KSlicer_GudhiCohomology0_i32
  elif is_vineyard == False and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.int64) and col.lower() == "INTRUSIVE_SET".lower() and "GudhiCohomology".lower() == pers_backend.lower():
    return _KSlicer_GudhiCohomology0_i64
  elif is_vineyard == False and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.float32) and col.lower() == "INTRUSIVE_SET".lower() and "GudhiCohomology".lower() == pers_backend.lower():
    return _KSlicer_GudhiCohomology0_f32
  elif is_vineyard == False and is_k_critical  ==  True and np.dtype(dtype) is np.dtype(np.float64) and col.lower() == "INTRUSIVE_SET".lower() and "GudhiCohomology".lower() == pers_backend.lower():
    return _KSlicer_GudhiCohomology0_f64
  elif is_vineyard == False and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.int32) and col.lower() == "INTRUSIVE_SET".lower() and "GudhiCohomology".lower() == pers_backend.lower():
    return _Slicer_GudhiCohomology0_i32
  elif is_vineyard == False and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.int64) and col.lower() == "INTRUSIVE_SET".lower() and "GudhiCohomology".lower() == pers_backend.lower():
    return _Slicer_GudhiCohomology0_i64
  elif is_vineyard == False and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.float32) and col.lower() == "INTRUSIVE_SET".lower() and "GudhiCohomology".lower() == pers_backend.lower():
    return _Slicer_GudhiCohomology0_f32
  elif is_vineyard == False and is_k_critical  ==  False and np.dtype(dtype) is np.dtype(np.float64) and col.lower() == "INTRUSIVE_SET".lower() and "GudhiCohomology".lower() == pers_backend.lower():
    return _Slicer_GudhiCohomology0_f64
  else:
    raise ValueError(f"Unimplemented combo for {pers_backend} : {is_vineyard=}, {is_k_critical=}, {dtype=}, {col=}")




def _hilbert_signed_measure(slicer, 
    vector[indices_type] degrees, 
    bool zero_pad=False, 
    indices_type n_jobs=0, 
    bool verbose=False,
    # bool expand_collapse=False, 
    # grid_conversion = None,
    bool ignore_inf = True,
    ):
  """
  Computes the signed measures given by the decomposition of the hilbert function.

  Input
  -----

   - simplextree:SimplexTreeMulti, the multifiltered simplicial complex
   - degrees:array-like of ints, the degrees to compute
   - n_jobs:int, number of jobs. Defaults to #cpu, but when doing parallel computations of signed measures, we recommend setting this to 1.
   - verbose:bool, prints c++ logs.
  
  Output
  ------
  
  `[signed_measure_of_degree for degree in degrees]`
  with `signed_measure_of_degree` of the form `(dirac location, dirac weights)`.
  """

  assert slicer.is_squeezed, "Squeeze grid first."
  if slicer.is_squeezed:
    grid_shape = np.array([len(f) for f in slicer.filtration_grid])
  else:
      grid_shape = (slicer.filtration_bounds()[1]).astype(python_indices_type)+1
  if zero_pad:
    for i, _ in enumerate(grid_shape):
      grid_shape[i] += 1 # adds a 0
  assert len(grid_shape) == slicer.num_parameters, "Grid shape size has to be the number of parameters."
  grid_shape_with_degree = np.asarray(np.concatenate([[len(degrees)], grid_shape]), dtype=python_indices_type)
  container_array = np.ascontiguousarray(np.zeros(grid_shape_with_degree, dtype=python_tensor_dtype).flatten())
  assert len(container_array) < np.iinfo(np.uint32).max, "Too large container. Raise an issue on github if you encounter this issue. (Due to tensor's operator[])"
  cdef vector[indices_type] c_grid_shape = grid_shape_with_degree
  cdef tensor_dtype[::1] container = container_array
  cdef tensor_dtype* container_ptr = &container[0]
  cdef signed_measure_type out = _compute_hilbert_sm(slicer, container_ptr, c_grid_shape, degrees,n_jobs, verbose, zero_pad, ignore_inf)
  pts, weights = np.asarray(out.first, dtype=python_indices_type).reshape(-1, slicer.num_parameters+1), np.asarray(out.second, dtype=python_tensor_dtype)
  slices = np.concatenate([np.searchsorted(pts[:,0], np.arange(degrees.size())), [pts.shape[0]] ])
  sms = [
      (pts[slices[i]:slices[i+1],1:],weights[slices[i]:slices[i+1]])
      for i in range(slices.shape[0]-1)
  ]
  return sms


## Rank invariant


## TODO : It is not necessary to do the Möbius inversion in python.
## fill rank in flipped death, then differentiate in cpp, then reflip with numpy.
def _rank_from_slicer(
        slicer, 
        vector[indices_type] degrees,
        bool verbose=False,
        indices_type n_jobs=1,
        bool zero_pad = False,
        grid_shape=None,
        bool plot=False,
        bool return_raw=False,
        bool ignore_inf = True,
        ):
    # cdef intptr_t slicer_ptr = <intptr_t>(slicer.get_ptr())
    if grid_shape is None:
        if slicer.is_squeezed:
          grid_shape = np.array([len(f) for f in slicer.filtration_grid])
        else:
            grid_shape = (slicer.filtration_bounds()[1]).astype(python_indices_type)+1
    grid_shape = np.asarray(grid_shape)

    cdef int num_parameters = len(grid_shape)

    # if zero_pad:
    #     grid_shape += 1
        # for i, _ in enumerate(grid_shape):
        #     grid_shape[i] += 1 # adds a 0
        # for i,f in enumerate(grid_conversion):
        #     grid_conversion[i] = np.concatenate([f, [mass_default[i]]])


    grid_shape_with_degree = np.asarray(np.concatenate([[len(degrees)], grid_shape, grid_shape]), dtype=python_indices_type)
    if verbose:
        print("Container shape: ", grid_shape_with_degree)
    container_array = np.ascontiguousarray(np.zeros(grid_shape_with_degree, dtype=python_tensor_dtype).ravel())
    assert len(container_array) < np.iinfo(python_indices_type).max, "Too large container. Raise an issue on github if you encounter this issue. (Due to tensor's operator[])"
    # if zero_pad:
    #     grid_shape_with_degree[1:] -= 1
    cdef vector[indices_type] c_grid_shape = grid_shape_with_degree
    cdef tensor_dtype[::1] container = container_array
    cdef tensor_dtype* container_ptr = &container[0]

    ## SLICERS
    if verbose:
        print("Computing rank invariant...", end="")
    _compute_rank_invariant(slicer, container_ptr, c_grid_shape, degrees, n_jobs, ignore_inf)
    if verbose:
        print("Done.")

    if verbose:
        print("Computing Möbius inversion...", end="")
    # if zero_pad:
    #     grid_shape_with_degree[1:] += 1
    rank = container_array.reshape(grid_shape_with_degree)
    rank = tuple(rank_decomposition_by_rectangles(rank_of_degree, threshold=zero_pad) for rank_of_degree in rank)
    if return_raw:
        return rank
    out = []
    def clean_rank(rank_decomposition):
        (coords, weights) = sparsify(np.ascontiguousarray(rank_decomposition))
        births = coords[:,:num_parameters]
        deaths = coords[:,num_parameters:]
        correct_indices = np.all(births<=deaths, axis=1)
        coords = coords[correct_indices]
        weights = weights[correct_indices]
        return coords, weights

    out = tuple(clean_rank(rank_decomposition) for rank_decomposition in rank)
    if verbose:
        print("Done.")
    return out
